//package kepler;

import java.awt.*;
import java.awt.Dimension;
import java.net.URL;
import java.net.MalformedURLException;
import java.io.*;
import kepler.*;
import kepler.view.*;
import kepler.object.*;
import kepler.control.*;
import utils.d2.*;
import utils.timers.*;
import utils.GUI.GridBagHelpers;
import utils.GUI.Bar;
import webwisdom.tango.consts.*;
import webwisdom.tango.messages.*;
import webwisdom.tango.interfaces.AppletBase;

public class SAKKepApp extends AppletBase implements EventHandles, EventQueueTarget, Dynamic
{
    KeplerView view;
    ConfigPanel config;
    GeneralPanel general;
    SweepPanel sweep;
    TimeAreaView taview;
    Planet earth;
    Star sun;
    TwoPerspective p;
    FixedKicker time;
    AnimationThread film,ofilm;
    // Reduce to handle events more frequently
    int slaveLeash = 2;            
    boolean running=false, tangoMaster=false, initDone=false, tangoKnowMaster=false;
    EventQueue eq;

    public final static byte TANGO_PAUSE             = 0;
    public final static byte TANGO_CONTINUE          = 1;
    public final static byte TANGO_START_SWEEPING    = 2;
    public final static byte TANGO_STOP_SWEEPING     = 3;
    public final static byte TANGO_ERASE_AREAS       = 4;
    public final static byte TANGO_ERASE_LINES       = 5;
    public final static byte TANGO_ZOOM_OUT          = 6;
    public final static byte TANGO_ZOOM_IN           = 7;
    public final static byte TANGO_SHOW_VEL          = 8;
    public final static byte TANGO_NO_SHOW_VEL       = 9;
    public final static byte TANGO_SHOW_ACC          = 10;
    public final static byte TANGO_NO_SHOW_ACC       = 11;
    public final static byte TANGO_SHOW_ORB          = 12;
    public final static byte TANGO_NO_SHOW_ORB       = 13;
    public final static byte TANGO_SSYSTEM_CONFIG    = 14;
    public final static byte TANGO_DRAW_NEW_CURVE    = 15;
    public final static byte TANGO_DRAW_ADD_TO_CURVE = 16;
    public final static byte TANGO_KICK_THE_KICKER   = 17;
  
    public void init() {
	tangoMaster = false;

	URL img_base=null;
	try{
	    img_base=new URL( getCodeBase(), "images");
	}
	catch(MalformedURLException mue) {
	    System.out.println(mue);
	    System.out.println( "Failed to locate images ");
	}
    
	setBackground(new Color(60,60,125));
	setForeground(Color.white);
	FixedKicker.dt=0.01;
	int viewW = 350;
	int viewH = 300;

	eq     = new EventQueue(this);
	time   = new FixedKicker(slaveLeash);
	sun    = new Star(new TwoVector(),1.0,img_base);
	earth  = new Planet(sun,1.0,0.5,img_base);
	p      = new TwoPerspective( new Dimension(viewW,viewH), 4.0);
	view   = new KeplerView(sun,earth,p);
    	config = new ConfigPanel();
	general= new GeneralPanel();
	sweep  = new SweepPanel();
    	taview = new TimeAreaView(earth);

	// Reduce to make timesteps come faster
	time.setSleepTime(125);
	time.addDriven(this,slaveLeash,true);
	time.addDriven(earth,1,false);

	/* xxx */
// 	time.setMaxKickCount(slaveLeash);

	view.resize(viewW,viewH);
	myLayout();
	initDone = true;

	try{
	    register();
	}catch (Exception e){
	    System.out.println(e);
	}
	time.start();

    }

    void myLayout() {
    	setLayout(new GridBagLayout());
	GridBagHelpers.insert(this,view,1,1,1,4,GridBagConstraints.BOTH,GridBagConstraints.NORTHWEST,0,0,1.0,1.0,8,8,0,0);
	GridBagHelpers.insert(this,general,2,1,1,1,GridBagConstraints.VERTICAL,GridBagConstraints.CENTER,0,0,0.0,1.0,8,0,5,8);
	GridBagHelpers.insert_fill(this,new Bar(),2,2,1,1,5,0,5,8);
	GridBagHelpers.insert(this,config,2,3,1,1,GridBagConstraints.VERTICAL,GridBagConstraints.CENTER,0,0,0.0,2.0,5,5,5,8);
	GridBagHelpers.insert(this,new Bar(),2,4,1,1,GridBagConstraints.HORIZONTAL,GridBagConstraints.SOUTH,0,0,0.0,0.0,5,0,5,8);
	GridBagHelpers.insert(this,taview,1,5,1,1,10,8,8,5,GridBagConstraints.CENTER);
	GridBagHelpers.insert(this,sweep,2,5,1,1,10,5,8,8,GridBagConstraints.CENTER);
    }

    void startThreads() {
	if(!running){
	    film=new AnimationThread(view,75);
	    ofilm=new AnimationThread(taview,75);
	    time.setActive(earth,true);
	    film.start();
	    ofilm.start();
	    running=true;
	}
    }

    void stopThreads() {
	if(running){
	    film.pleaseStop();
	    ofilm.pleaseStop();
	    time.setActive(earth,false);
	    running=false;
	}
    }

    public boolean handleEvent(Event e){
	if(tangoMaster && tangoKnowMaster){
	    tangoDistributeEvent(e);
	    eq.addEvent(e);
	}
	return(true);
    }

    public void timestep(double dt){
	if(tangoMaster && tangoKnowMaster){
	    byte b[] = {TANGO_KICK_THE_KICKER};
	    send(new AppEventMessage(Const.DATA,b));
	    eq.empty();

	    /* xxx */
// 	    time.again();
	}

    }	
	    
    public void acceptEvent(Event e) {
	if(!initDone) return;
	if(Event.ACTION_EVENT == e.id) {
	    if(e.arg instanceof String) {
		String sarg = (String)e.arg;
		if(PAUSE.equals(sarg)) {
		    stopThreads();
		    return;
		}else if(CONTINUE.equals(sarg)) {
		    startThreads(); 
		    return;
		}else if(START_SWEEPING.equals(sarg)) {
		    earth.startSweeping();
		    distributeEvent(e);
		    return;
		}else if( STOP_SWEEPING.equals(sarg)) {
		    earth.stopSweeping();
		    distributeEvent(e);
		    return;
		}else if(ERASE_AREAS.equals(sarg)) {
		    earth.stopSweeping();
		    distributeEvent(e);
		    if(running && !film.isAlive()) view.draw();
		    return;
		}else if(ERASE_LINES.equals(sarg)) {
		    distributeEvent(e);
		    if( !running ) view.draw();
		    return;
		}else if(ZOOM_IN.equals(sarg)     ||
			 ZOOM_OUT.equals(sarg)    ||
			 SHOW_VEL.equals(sarg)    ||
			 NO_SHOW_VEL.equals(sarg) ||
			 SHOW_ACC.equals(sarg)    ||
			 SHOW_ORB.equals(sarg)    ||
			 NO_SHOW_ORB.equals(sarg) ||
			 NO_SHOW_ACC.equals(sarg)){
		    distributeEvent(e);
		    return;
		}
		else{
		    if(!distributeEvent(e)) super.handleEvent(e);
		    return;
		}
	    }else if(e.arg instanceof SSystemConfig) {
		stopThreads();
		acceptEvent(new Event(this,Event.ACTION_EVENT,STOP_SWEEPING));
		distributeEvent(new Event(this,Event.ACTION_EVENT,ERASE_AREAS));
		SSystemConfig ss=(SSystemConfig)e.arg;
		sun.setMass(ss.stellarmass);
		earth.defineOrbit(sun,ss.semimajor,ss.eccentricity);
		distributeEvent(e);
		return;
	    }
	}else if(Event.MOUSE_DOWN == e.id){
	    if(e.arg instanceof String && DRAW_NEW_CURVE.equals((String)e.arg)){
		if( !running ) view.draw();
		distributeEvent(e);
		return;
	    }
	    if(!distributeEvent(e)) super.handleEvent(e);
	}else if(Event.MOUSE_DRAG == e.id){
	    if(e.arg instanceof String && DRAW_ADD_TO_CURVE.equals((String)e.arg)){
		if(!running){
		    view.draw();
		}
		distributeEvent(e);
		return;
	    }
	    if(!distributeEvent(e)) super.handleEvent(e);
	}
 	if(!distributeEvent(e)) super.handleEvent(e);
	return;
    }


    public void receive(AppEventMessage aem){
	if(!initDone) return;
	if(Const.CONTROL == aem.getType()){
	    CAppAnswerEventMessage mes =  (CAppAnswerEventMessage) aem;
	    if(CAConsts.ISMASTER == mes.getAnswerType()){
		try{
		    if(mes.isMaster()){
			tangoMaster = true;
			/* xxx */
			time.setMaxKickCount(0);
		    }else{
			tangoMaster = false;
			/* xxx */
			time.setMaxKickCount(slaveLeash);
		    }
		}catch(Exception e){
		    System.out.println("time: "+time);
		    System.out.println("e: "+e);
		}
		tangoKnowMaster = true;
	    }
	}else{
	    if(!tangoKnowMaster) return;
	    byte[] mes = aem.getData();
	    switch(mes[0]){
	    case TANGO_PAUSE:
		eq.addEvent(new Event(this,Event.ACTION_EVENT,PAUSE));
		break;
	    case TANGO_CONTINUE:
		eq.addEvent(new Event(this,Event.ACTION_EVENT,CONTINUE));
		break;
	    case TANGO_START_SWEEPING:   
		eq.addEvent(new Event(this,Event.ACTION_EVENT,START_SWEEPING));
		break;
	    case TANGO_STOP_SWEEPING:    
		eq.addEvent(new Event(this,Event.ACTION_EVENT,STOP_SWEEPING));
		break;
	    case TANGO_ERASE_AREAS:      
		eq.addEvent(new Event(this,Event.ACTION_EVENT,ERASE_AREAS));
		break;
	    case TANGO_ERASE_LINES:      
		eq.addEvent(new Event(this,Event.ACTION_EVENT,ERASE_LINES));
		break;
	    case TANGO_ZOOM_OUT:         
		eq.addEvent(new Event(this,Event.ACTION_EVENT,ZOOM_OUT));
		break;
	    case TANGO_ZOOM_IN:          
		eq.addEvent(new Event(this,Event.ACTION_EVENT,ZOOM_IN));
		break;
	    case TANGO_SHOW_VEL:         
		eq.addEvent(new Event(this,Event.ACTION_EVENT,SHOW_VEL));
		break;
	    case TANGO_NO_SHOW_VEL:      
		eq.addEvent(new Event(this,Event.ACTION_EVENT,NO_SHOW_VEL));
		break;
	    case TANGO_SHOW_ACC:         
		eq.addEvent(new Event(this,Event.ACTION_EVENT,SHOW_ACC));
		break;
	    case TANGO_NO_SHOW_ACC:      
		eq.addEvent(new Event(this,Event.ACTION_EVENT,NO_SHOW_ACC));
		break;
	    case TANGO_SHOW_ORB:         
		eq.addEvent(new Event(this,Event.ACTION_EVENT,SHOW_ORB));
		break;
	    case TANGO_NO_SHOW_ORB:      
		eq.addEvent(new Event(this,Event.ACTION_EVENT,NO_SHOW_ORB));
		break;
	    case TANGO_SSYSTEM_CONFIG:{
		ByteArrayInputStream bais = new ByteArrayInputStream(mes);
		bais.skip(1);
		DataInputStream dis = new DataInputStream(bais);
		SSystemConfig ssc = new SSystemConfig();
		try{
		    ssc.stellarmass = dis.readDouble();
		    ssc.semimajor = dis.readDouble();
		    ssc.eccentricity = dis.readDouble();
		}catch(java.io.IOException ioe){
		}
		eq.addEvent(new Event(this,Event.ACTION_EVENT,ssc));
		break;}
	    case TANGO_DRAW_NEW_CURVE:{
		ByteArrayInputStream bais = new ByteArrayInputStream(mes);
		bais.skip(1);
		DataInputStream dis = new DataInputStream(bais);
		Event e = new Event(this,Event.MOUSE_DOWN,DRAW_NEW_CURVE);
		try{
		    e.x = dis.readInt();
		    e.y = dis.readInt();
		}catch(java.io.IOException ioe){
		}
		eq.addEvent(e);
		break;}
	    case TANGO_DRAW_ADD_TO_CURVE:{
		ByteArrayInputStream bais = new ByteArrayInputStream(mes);
		bais.skip(1);
		DataInputStream dis = new DataInputStream(bais);
		Event e = new Event(this,Event.MOUSE_DRAG,DRAW_ADD_TO_CURVE);
		try{
		    e.x = dis.readInt();
		    e.y = dis.readInt();
		}catch(java.io.IOException ioe){
		}
		eq.addEvent(e);
		break;}
	    case TANGO_KICK_THE_KICKER:{
		time.kick();
		eq.empty();
		time.again();
	    }
	    }
	}
    }


    public void tangoDistributeEvent(Event e) {
	AppEventMessage aem;

	if(Event.ACTION_EVENT == e.id) {
	    if(e.arg instanceof String) {
		String sarg = (String)e.arg;
		if(PAUSE.equals(sarg)) {
		    byte b[]= {TANGO_PAUSE};
		    aem = new AppEventMessage(Const.DATA,b);
		    send(aem);
		}
		else if(CONTINUE.equals(sarg)) {
		    byte b[]= {TANGO_CONTINUE};
		    aem = new AppEventMessage(Const.DATA,b);
		    send(aem);
		}
		else if(START_SWEEPING.equals(sarg)) {
		    byte b[]= {TANGO_START_SWEEPING};
		    aem = new AppEventMessage(Const.DATA,b);
		    send(aem);
		}
		else if(STOP_SWEEPING.equals(sarg)) {
		    byte b[]= {TANGO_STOP_SWEEPING};
		    aem = new AppEventMessage(Const.DATA,b);
		    send(aem);
		}
		else if(ERASE_AREAS.equals(sarg)) {
		    byte b[]= {TANGO_ERASE_AREAS};
		    aem = new AppEventMessage(Const.DATA,b);
		    send(aem);
		}
		else if(ERASE_LINES.equals(sarg)) {
		    byte b[]= {TANGO_ERASE_LINES};
		    aem = new AppEventMessage(Const.DATA,b);
		    send(aem);
		}
		else if(ZOOM_IN.equals(sarg)) {
		    byte b[]= {TANGO_ZOOM_IN};
		    aem = new AppEventMessage(Const.DATA,b);		    
		    send(aem);
		}
		else if(ZOOM_OUT.equals(sarg)){
		    byte b[]= {TANGO_ZOOM_OUT};
		    aem = new AppEventMessage(Const.DATA,b);		    
		    send(aem);
		}
		else if(SHOW_VEL.equals(sarg)){
		    byte b[]= {TANGO_SHOW_VEL};
		    aem = new AppEventMessage(Const.DATA,b);		    
		    send(aem);
		}
		else if(NO_SHOW_VEL.equals(sarg)){
		    byte b[]= {TANGO_NO_SHOW_VEL};
		    aem = new AppEventMessage(Const.DATA,b);		    
		    send(aem);
		}
		else if(SHOW_ACC.equals(sarg)){
		    byte b[]= {TANGO_SHOW_ACC};
		    aem = new AppEventMessage(Const.DATA,b);		    
		    send(aem);
		}
		else if(NO_SHOW_ACC.equals(sarg)){
		    byte b[]= {TANGO_NO_SHOW_ACC};
		    aem = new AppEventMessage(Const.DATA,b);		    
		    send(aem);
		}
		else if(SHOW_ORB.equals(sarg)){
		    byte b[]= {TANGO_SHOW_ORB};
		    aem = new AppEventMessage(Const.DATA,b);		    
		    send(aem);
		}
		else if(NO_SHOW_ORB.equals(sarg)){
		    byte b[]= {TANGO_NO_SHOW_ORB};
		    aem = new AppEventMessage(Const.DATA,b);		    
		    send(aem);
		}
	    }else if(e.arg instanceof SSystemConfig) {
		SSystemConfig ssc = (SSystemConfig)(e.arg);
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		DataOutputStream dos = new DataOutputStream(baos);
		try{
		    dos.writeByte(TANGO_SSYSTEM_CONFIG);
		    dos.writeDouble(ssc.stellarmass);
		    dos.writeDouble(ssc.semimajor);
		    dos.writeDouble(ssc.eccentricity);
		}catch(java.io.IOException ioe){
		}
		aem = new AppEventMessage(Const.DATA, baos.toByteArray());
		send(aem);
	    }
	}else if(Event.MOUSE_DOWN == e.id){
	    if(e.arg instanceof String && DRAW_NEW_CURVE.equals((String)e.arg)){
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		DataOutputStream dos = new DataOutputStream(baos);
		try{
		    dos.writeByte(TANGO_DRAW_NEW_CURVE);
		    dos.writeInt(e.x);
		    dos.writeInt(e.y);
		}catch(java.io.IOException ioe){
		}
		aem = new AppEventMessage(Const.DATA,baos.toByteArray());
		send(aem);
	    }
	}else if(Event.MOUSE_DRAG == e.id){
	    if(e.arg instanceof String && DRAW_ADD_TO_CURVE.equals((String)e.arg)){
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		DataOutputStream dos = new DataOutputStream(baos);
		try{
		    dos.writeByte(TANGO_DRAW_ADD_TO_CURVE);
		    dos.writeInt(e.x);
		    dos.writeInt(e.y);
		}catch(java.io.IOException ioe){
		}
		aem = new AppEventMessage(Const.DATA,baos.toByteArray());
		send(aem);
	    }
	}
    }


    boolean distributeEvent(Event e){
	boolean tmp=true;
	tmp=view.readEvent(e)|taview.readEvent(e)|general.readEvent(e)|sweep.readEvent(e)|config.readEvent(e);
	return tmp;
    }

    public boolean readEvent(Event e){
	return handleEvent(e);
    }
  
    public void stop(){ stopThreads(); }
}










    
