package kepler.control;

import java.awt.*;
import utils.GUI.GridBagHelpers;


class StatusCanvas extends Canvas{
    final int diskdiameter=5;
    final String r="Ready";
    final String s="Sweeping";
    int stringx,circlex,string1y,string2y,circle1y,circle2y;

    boolean sweeping;

    StatusCanvas(){ 
	super(); 
	setFont(new Font("Helvetica",Font.PLAIN,8)); 
	sweeping=false;
    }
  
    public void addNotify(){
	super.addNotify();
	measure();
    }

    void measure(){
	Dimension d=size();
	FontMetrics fm=getFontMetrics(getFont());
	if(d==null || fm== null ) return;
    
	int centery=size().height/2;
    
	string1y=centery-3;
	string2y=centery+3+fm.getHeight();
	stringx=20;
	circlex=8;
	circle1y=string1y-fm.getHeight()/2;
	circle2y=string2y-fm.getHeight()/2;
    }

    public Dimension minimumSize(){
	if( stringx==0) measure();
	Dimension tmp=new Dimension( getFontMetrics(getFont()).stringWidth(s)+30,
				     getFontMetrics(getFont()).getHeight()+26);
	return tmp;
    }

    public void reshape(int x,int y,int width,int height){
	super.reshape(x,y,width,height);
	measure();
    }

    public void paint(Graphics g){
	if(sweeping){
	    g.setColor(Color.red);
	    g.fillOval(circlex,circle1y,diskdiameter,diskdiameter);
	}
	else{
	    g.setColor(Color.green);
	    g.fillOval(circlex,circle2y,diskdiameter,diskdiameter);
	}
	g.setColor(getForeground());
	g.drawOval(circlex,circle1y,diskdiameter,diskdiameter);
	g.drawOval(circlex,circle2y,diskdiameter,diskdiameter);
	g.drawString(s,stringx,string1y);
	g.drawString(r,stringx,string2y);
    }
    public void startSweep(){
	sweeping=true;
	repaint();
    }
    public void stopSweep(){
	sweeping=false;
	repaint();
    }
}
    
public class SweepPanel extends Panel implements kepler.EventHandles{
    Button start,stop;
    Button rmareas;
    StatusCanvas sc;

    public SweepPanel(){
	super();
	start=new Button(START_SWEEPING);
	stop=new Button(STOP_SWEEPING);
	rmareas=new Button(ERASE_AREAS);
	sc=new StatusCanvas();
	sc.resize(sc.minimumSize());
	setLayout(new GridBagLayout());

	GridBagHelpers.insert(this,start,
			      1,1,
			      1,1
			      );
  
	GridBagHelpers.insert(this,sc,
			      2,2,
			      1,1,
			      GridBagConstraints.CENTER
			      );
			
	GridBagHelpers.insert(this,stop,
			      1,2,
			      1,1,
			      GridBagConstraints.CENTER
			      );
	GridBagHelpers.insert(this,rmareas,
			      1,3,
			      1,1,
			      GridBagConstraints.CENTER
			      );
			
	stop.disable();
    }


    public boolean handleEvent(Event e){
	//Button presses already have String args
	e.target=this;
	return super.handleEvent(e);
    }

    public boolean readEvent(Event e){
	if( e.id == Event.ACTION_EVENT ){
	    Object arg = e.arg;
	    if(START_SWEEPING.equals(arg)){
		sc.startSweep();
		stop.enable();
		start.disable();
		stop.requestFocus();
	    }else if(STOP_SWEEPING.equals(arg)){
		sc.stopSweep();
		start.enable();
		stop.disable();
		start.requestFocus();
	    }else if(ERASE_AREAS.equals(arg)){
		handleEvent( new Event(null,Event.ACTION_EVENT,STOP_SWEEPING));
	    }
	    return false;
	}
	return false;
    }
}








