package kepler.object;

import utils.d2.*;
import utils.d2.realvisible.Disk;
import utils.timers.Dynamic;
import kepler.object.Star;
import utils.image.PlanetImage;
import java.awt.Image;
import java.net.URL;
import java.net.MalformedURLException;

public class Planet extends Disk 
    implements Dynamic,java.awt.image.ImageObserver{

    public static final double G=4.0*Math.PI*Math.PI;

    double e;  // eccentricity of orbit
    double ep; // derived from eccentricity and semi-major axis
    double l;  // orbital angular momentum per unit mass 
    double theta;  // current angle of planet wrt position at perihelion
    double R;      //  current distance from the sun
    Star sun;

    double time,area; // current time/area swept out
    boolean sweeping; // should i be keeping track of the amount of 
    // time/area swept out
    TwoVector v;
    Image I;



    public Planet(Star s, double semi, double ecc){
	super();
	defineOrbit(s,semi,ecc);
	setSize(0.00464*10);
	time=0.0;
	area=0.0;
	sweeping=false;
	v=new TwoVector();
	try{
	    I=PlanetImage.makeImage("Earth");
	} 
	catch( java.net.MalformedURLException mue){
	    System.out.println("error getting image for planet");
	    I=null;
	}
    }


    public Planet(Star s, double semi, double ecc,URL image_base){
	super();
	defineOrbit(s,semi,ecc);
	setSize(0.00464*10);
	time=0.0;
	area=0.0;
	sweeping=false;
	v=new TwoVector();
	try{
	    I=PlanetImage.makeImage(image_base,"Earth");
	} 
	catch( java.net.MalformedURLException mue){
	    System.out.println("error getting image for planet "+image_base);
	    I=null;
	}
    }

    public Planet(Star s,double semi,double ecc,TwoPerspective p){
	this(s,semi,ecc);
	updateDisplay(p);
    }

    public void defineOrbit(Star s,double semi,double ecc){
	ep=semi*(1.0-ecc*ecc);
	e=ecc;
	l=Math.sqrt(G*s.getMass()*ep);
	theta=0.0;
	double dumr=ep/(1.0-e*Math.cos(theta));
	TwoVector r=TwoVector.new_rt(dumr,theta);
	R=r.dot(r);
	setLocation( r.plus(s.getLocation()));
	sun=s;
    }

    public void timestep(double dt){
	TwoVector oldloc=getLocation();
	double dtheta=l/R*dt;
	theta+=dtheta;
	if(sweeping){
	    time+=dt;
	    area+=R*dtheta/2.0;
	}
	double dumr=ep/(1.0-e*Math.cos(theta));
	TwoVector r=TwoVector.new_rt(dumr,theta);
	R=r.dot(r);
	setLocation( r.plus(sun.getLocation()));
	if( dt!=0.0) v= ( getLocation().minus(oldloc)).times(1.0/dt);
	else v=new TwoVector();
    }

    public void startSweeping(){
	sweeping=true;
    }
    public void stopSweeping(){
	sweeping=false;
	time=0.0; area=0.0;
    }
    public double getTime(){ return time; }
    public double getArea(){ return area; }
    public boolean isSweeping(){ return sweeping; }
    public TwoVector getV(){
	return TwoVector.new_xy( -l/ep*Math.sin(theta), l/ep*Math.cos(theta)-e*l/ep);
    }
    public TwoVector getA(){
	TwoVector tmp=getLocation().minus( sun.getLocation()).normalized();
	tmp.timesEquals(-l*l/ep/(R*R));
	return tmp;
    }

    public TwoVector getLocAt(double th){
	double dumr=ep/(1.0-e*Math.cos(th));
	TwoVector r=TwoVector.new_rt(dumr,th);
	r.plusEquals(sun.getLocation());
	return r;
    }
    public TwoVector getXmax(){ return getLocAt(0.0); }
    public TwoVector getXmin(){ return getLocAt( Math.PI); }
    public TwoVector getYmin(){ return getLocAt( Math.acos(e)); }
    public TwoVector getYmax(){ return getLocAt( -Math.acos(e)); }
   
    public void draw(java.awt.Graphics g){
	g.drawImage(I,ULcorner.x,ULcorner.y,diameter,diameter,this);
    }

    public boolean imageUpdate(Image img,int infoflags,int x,int y,
			       int width,int height){
	try{
	    I=PlanetImage.makeImage("Earth");
	} 
	catch( java.net.MalformedURLException mue){
	    System.out.println("error getting image for planet ");
	    I=null;
	}
	return true;
    }
}
