package kepler.view;

import kepler.object.*;
import java.awt.*;
import kepler.control.SSystemConfig;
import utils.timers.Animated;
import utils.d2.TwoPerspective;
import utils.d2.realvisible.FilledColoredMultigon;
import utils.d2.realvisible.ThinArrow;
import utils.GUI.ColorHelper;
import utils.image.PlanetImage;
import utils.GUI.drawing.SetOfCurves;

public class KeplerView extends Canvas 
    implements Animated,kepler.view.ViewConfig,kepler.EventHandles, Runnable{
  
    Star sun;
    Planet earth;
    TwoPerspective p;

    Graphics offG;
    Image    offI;
    Dimension mysize;
  
    FilledColoredMultigon[] areas;
    int currentarea;			
    ThinArrow v;	
    ThinArrow a;
    double factor;			
    Rectangle orbit;
    SetOfCurves drawing;

    public KeplerView(Star s,Planet e,TwoPerspective tp){
	sun=s;
	earth=e;
	p=tp;
	sun.updateDisplay(p);
	earth.updateDisplay(p);
	setBackground(Color.black);
	mysize=size();
	areas=new FilledColoredMultigon[NUM_AREAS];
	for(int i=0;i<NUM_AREAS;++i){
	    areas[i]=new FilledColoredMultigon(ColorHelper.blackPalette(i));
	}
	currentarea=-1;
	v=null;
	factor=0.05;
	orbit=null;
	drawing=new SetOfCurves();

	Thread kicker=new Thread(this);
	kicker.start();
    }

    public void paint(Graphics g){
	for(int i=0;i<NUM_AREAS;++i) areas[i].draw(g);
	g.setColor(Color.white);
	if(orbit!=null) g.drawOval(orbit.x,orbit.y,orbit.width,orbit.height);
	g.setColor(Color.blue);
	earth.draw(g);
	if(v!=null){
	    v.draw(g);
	}
	if( a!=null){
	    a.draw(g);
	}
  
	sun.draw(g);
	g.setColor(Color.white);
	drawing.draw(g);

    }

    public void update(Graphics g){
	if(p.hasChanged()){
	    if( orbit!=null) orbit=setUpOrbit();
	    sun.updateDisplay(p);
	    for(int i=0;i<NUM_AREAS;++i) areas[i].updateDisplay(p);
	}
	earth.updateDisplay(p);
	if( v!=null){
	    v.setLocation(earth.getLocation());
	    v.setDirection(earth.getV().times(vfactor));
	    v.updateDisplay(p);
	}
	if( a!=null){
	    a.setLocation(earth.getLocation());
	    a.setDirection(earth.getA().times(afactor));
	    a.updateDisplay(p);
	}
	p.clearChanged();
	super.update(g);
    }

    public void draw(){
	Dimension d=size();
	if( offG==null || offI==null ||
	    d.width!=mysize.width || d.height!=mysize.height){
	    mysize=d;
      System.out.println("creating image0");
	    offI=createImage(mysize.width,mysize.height);
	    offG=offI.getGraphics();
	}

	if(earth.isSweeping() && currentarea<NUM_AREAS  ) 
	    areas[currentarea].addPoint(earth.copyOfLocation(),
					p);
	update(offG);
	getGraphics().drawImage(offI,0,0,this);
    }


    Rectangle setUpOrbit(){
	Rectangle tmp=new Rectangle();
	Point xmin,ymin,ymax,xmax;
	xmin=p.getDisplay( earth.getXmin() );
	ymin=p.getDisplay( earth.getYmin() );
	xmax=p.getDisplay( earth.getXmax() );
	ymax=p.getDisplay( earth.getYmax() );
	
	tmp.x=xmin.x;
	tmp.y=ymin.y;
	tmp.width=xmax.x-xmin.x;
	tmp.height=ymax.y-ymin.y;
	return tmp;
    }  


    public void run(){
	while( !PlanetImage.doneLoading() ){
	    try{
		Thread.currentThread().sleep(1000);
		repaint();
	    }
	    catch(InterruptedException ie){
		repaint();
		return;
	    }
	}
    }

    public boolean handleEvent(Event e){
	if(e.id == Event.MOUSE_DOWN){
	    e.target = this;
	    e.arg = DRAW_NEW_CURVE;
	    return false;
	}else if(e.id == Event.MOUSE_DRAG){
	    e.target = this;
	    e.arg = DRAW_ADD_TO_CURVE;
	    return false;
	}
	return(super.handleEvent(e));
    }

    public boolean readEvent(Event e){
	Object arg = e.arg;
	if( arg instanceof String) return stringAction(e,(String)arg);
	else if( arg instanceof SSystemConfig ){
	    if(orbit!=null){
		orbit=setUpOrbit();
	    }
	    repaint();
	    p.setChanged();
	    return false;
	}
	return(false);
    }

    boolean stringAction(Event e,String sarg){
	if(ZOOM_IN.equals(sarg)){
	    p.magnify(2.0); draw();
	    return false;
	}else if(ZOOM_OUT.equals(sarg)){
	    p.magnify(0.5);
	    draw();
	    return false;
	}else if(START_SWEEPING.equals(sarg)  ){
	    ++currentarea;
	    if(currentarea<NUM_AREAS){
		areas[currentarea].addPoint(sun.copyOfLocation(),p);
		areas[currentarea].addPoint( earth.copyOfLocation(),p);
	    }
	    repaint();
	    return false;
	}
	else if(ERASE_AREAS.equals(sarg)  ){
	    for(int i=0;i<NUM_AREAS;++i) areas[i].removeAllPoints();
	    currentarea=-1;
	    repaint();
	    return false;
	}
	else if( SHOW_VEL.equals(sarg) ){
	    v=new ThinArrow(earth.getLocation(),earth.getV().times(vfactor),p);
	    v.setColor(VEL_COLOR);
	    repaint();
	    return false;
	}
	else if( NO_SHOW_VEL.equals(sarg)){
	    v=null;
	    repaint();
	    return false;
	}
	else if( SHOW_ACC.equals(sarg) ){
	    a=new ThinArrow(earth.getLocation(),earth.getA().times(afactor),p);
	    a.setColor(ACC_COLOR);
	    repaint();
	    return false;
	}
	else if( NO_SHOW_ACC.equals(sarg)){
	    a=null;
	    repaint();
	    return false;
	}
    	else if( SHOW_ORB.equals(sarg)){
	    orbit=setUpOrbit();
	    repaint();
	    return false;
	}
	else if( NO_SHOW_ORB.equals(sarg)){
	    orbit=null;
	    repaint();
	    return false;
	}
	else if( ERASE_LINES.equals(sarg)){
	    drawing=new SetOfCurves();
	    return false;
	}else if(DRAW_NEW_CURVE.equals(sarg)){
	    drawing.newCurve();
	    drawing.addPoint(e.x,e.y);
	    return false;
	}else if(DRAW_ADD_TO_CURVE.equals(sarg)){
	    drawing.addPoint(e.x,e.y);
	    return false;
	}
	else return false;
    }
}













