package kepler.view;

import java.awt.*;
import utils.GUI.ColorHelper;
import kepler.object.Planet;
import utils.timers.Animated;
import utils.GUI.GridBagHelpers;
import utils.GUI.DoubleTextDisplay;

class TimeAreaPanel extends Panel implements kepler.view.ViewConfig{
    DoubleTextDisplay time,area;
  
    public TimeAreaPanel(){
	time=new DoubleTextDisplay( TIME_COLUMNS);
	area=new DoubleTextDisplay( TIME_COLUMNS);

	time.setValue(0.0);
	area.setValue(0.0);
	setLayout(new GridLayout(2,1));
	add(time); add(area);
    }

    public void setValues(Planet p){
	setValues( p.getTime(), p.getArea() );
    }
    public void setValues(double tt,double aa){
	time.setValue( tt );
	area.setValue( aa );
    }
    public double getTime(){ return time.getValue();}
    public double getArea(){ return area.getValue(); }
}


public class TimeAreaView extends Panel 
    implements Animated,kepler.view.ViewConfig,
	       kepler.EventHandles{
    TimeAreaPanel[] row;
    int numfill;
    Planet pl;

    public TimeAreaView(Planet p){
	row=new TimeAreaPanel[NUM_AREAS];
	setLayout(new GridBagLayout());
	GridBagHelpers.insert(this,new Label("Time"),
			      1,1,
			      1,1,
			      GridBagConstraints.CENTER
			      );
	GridBagHelpers.insert(this,new Label("Area"),
			      1,2,
			      1,1,
			      GridBagConstraints.CENTER
			      );
	for(int i=0;i<NUM_AREAS;++i){

	    row[i]=new TimeAreaPanel();
	    row[i].setForeground( ColorHelper.blackPalette(i));
	    GridBagHelpers.insert(this,row[i],
				  2+i,1,
				  1,2
				  );
	}
	numfill=-1;
	pl=p;
    }


    public void draw(){
	if(pl.isSweeping() && numfill<NUM_AREAS && numfill>=0){
	    row[numfill].setValues(pl);	
	    if(numfill>0){	
		if( row[numfill].getTime() >= row[0].getTime() ){
		    pl.stopSweeping();
		    postEvent(new Event( null,Event.ACTION_EVENT,STOP_SWEEPING));
		}
	    } 
	}
    }

    public boolean handleEvent(Event e){
	e.target=this;
	return super.handleEvent(e);
    }
    public boolean readEvent(Event e){
	if( e.target != this && e.id == Event.ACTION_EVENT ){
	    if(e.arg instanceof String ) return stringAction(e,(String)(e.arg));
	    else return super.action(e,e.arg);
	}
	else return false;
    }


    synchronized boolean stringAction(Event e,String sarg){
	if(START_SWEEPING.equals(sarg)){ ++numfill; return true; }
	else if(ERASE_AREAS.equals(sarg)){ 
	    numfill=-1; 
	    for(int i=0;i<NUM_AREAS;++i) row[i].setValues(pl);
	    return true;
	}
	else return false;
    }
}









