package utils.GUI;

import java.awt.*;


/************************************************
 * provides a graphical means to show an angle,
 * basically as an arrow on a circle.
 * I have chosen to work in radians
 ************************************************/

public class AngleDisplay extends Canvas{
  static final int HEADLENGTH=12;
  static final double C=Math.cos( Math.PI/6.0);
  static final double S=Math.sin( Math.PI/6.0);


  Point center; // the center of the canvas
  int rad;      // the display radius of the arrow
  double value; // the angle

  Point tip;
  Point tail;
  
  Polygon head;

  Color pointer_color;
  Color field_color;


  /**
   * construct angle display with given initial orientation
   **/
  public AngleDisplay(double init_val){
    center=new Point(0,0);
    rad=0;
    value=init_val;
    tip=new Point(0,0);
    tail=new Point(0,0);
    head =new Polygon();
    for(int i=0;i<3;++i){
      head.addPoint(0,0);
    }
    field_color=Color.white;
    pointer_color=Color.black;
  }

  public AngleDisplay(){
    this(0.0);
  }

  /**
   * sets the direction of the arrow
   * this does force repaint
   **/
  public void setValue(double new_ang){
    value=new_ang;
    repaint();
  }
  public double getValue(){ return value; }
  
  /**
   * sets the color of the pointer
   * does not force repaint
   **/
  public void setPointerColor(Color c){ pointer_color=c; }
  public Color getPointerColor(){ return pointer_color; }
  
  public Color getFieldColor(){ return field_color; }
  /**
   * sets the background of the dial
   * does not force repaint
   **/
  public void setFieldColor(Color c){ field_color=c; }

  public void resize(int width,int height){
    super.resize(width,height);
    center.x=width/2; center.y=height/2;
    int smaller=Math.min(width,height);
    rad=smaller/2;
  }


  public Dimension minimumSize(){
    return new Dimension(16,16);
  }

  public Dimension preferredSize(){
    return new Dimension(20,20);
  }
      



  void updatePointer(){
    double c=Math.cos(value);
    double s=Math.sin(value);
    int cr=(int)(Math.round(rad*c));
    int sr=(int)(Math.round( rad*s));
    tip.x=center.x+cr;
    tip.y=center.y-sr;
    
    tail.x=center.x-cr;
    tail.y=center.y+sr;


    
    head.xpoints[2]=tip.x;
    head.ypoints[2]=tip.y;
    
    cr=(int)(Math.round( HEADLENGTH*c*C));
    sr=(int)(Math.round( HEADLENGTH*s*S));

    head.xpoints[0]=tip.x-cr-sr;
    head.xpoints[1]=tip.x-cr+sr;
    
    cr=(int)(Math.round( HEADLENGTH*s*C));
    sr=(int)(Math.round( HEADLENGTH*c*S));
    
    head.ypoints[0]=tip.y+cr-sr;
    head.ypoints[1]=tip.y+cr+sr;
    
  }

  public void update(Graphics g){
    updatePointer();
    super.update(g);
  }
  
  public void paint(Graphics g){
    
    if( tail.x==0)updatePointer();
    g.setColor( field_color);
    g.fillOval( center.x-rad,center.y-rad, 2*rad,2*rad);
    
    g.setColor( pointer_color);
    g.drawLine( tip.x,tip.y,tail.x,tail.y);
    g.drawLine( tip.x-1,tip.y,tail.x-1,tail.y);
    g.drawLine( tip.x,tip.y+1,tail.x,tail.y+1);
    g.fillPolygon( head );
  }

}
