package utils.GUI;

import java.awt.Color;
import java.awt.Event;
import java.awt.Graphics;
import java.awt.Dimension;


/***************************************************************
 * A GUI component which allows the user to select a color.
 * it generates ACTION_EVENTS with an arg which is the Color
 * that the user has selected
 ***************************************************************/

public class ColorChooser extends java.awt.Canvas{
  
  public static int MIN=12;

  Color[] hues;
  int dx;
  int current;
  
  /**
   * specify the palette as an array of Colors
   **/
  public ColorChooser(Color[] c){
    super();
    hues=c;
    current=0;
  }

  public void addNotify(){
    super.addNotify();
    dx=size().width/hues.length;
  }

  
  public Dimension minimumSize(){
    return new Dimension( MIN*hues.length,MIN);
  }
  public Dimension preferredSize(){
    return new Dimension( 2*MIN*hues.length,2*MIN);
  }

  public void resize(int w,int h){
    super.resize(w,h);
    dx=size().width/hues.length;
  }

  /**
   * generates ACTION_EVENTS with an arg which is the
   * chosen color
   **/
  public boolean mouseDown(Event e,int x,int y){
    
    current=x/dx;
    postEvent( new Event(this,Event.ACTION_EVENT,hues[current] ) );
    repaint();
    return true;
    
  }


  public void paint(Graphics g){
    int x=0;
    int dy=size().height;

    for(int i=0;i<hues.length;++i){
      g.setColor( hues[i] );
      g.fillRect( x,0,dx,dy);
      if( i==current){
	g.setColor(Color.black);
	g.drawLine( x,0,x+dx,dy);
	g.drawLine( x+dx,0,x,dy);
      }
      x+=dx;
    }
  }
  
  public Color getCurrentColor(){ return hues[current]; }

}

