package utils.GUI;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Graphics;


/*****************************************************************
 * a class which allows one to draw formatted doubles
 * onto a component. cf. DoubleTextDisplay
 ****************************************************************/


public class DoubleGraphicsDisplay{
  public static final String EMPTY="";
  int sig;
  int abs_size;
  
  FontMetrics abs_fm; //font metrics for the abssica
  FontMetrics exp_fm; // font metrics for the exponent
  Font exp_fn;    // the font for the exponent
  Font abs_fn;

  int exp_y;  // how far up to place the exponent
  int exp_x;

  Point llCorner; // the lower left corner of the text display

  double value;
  String abs;
  String exp;

  /**
   * @param v -- the value to display
   * @param sigfig -- number of significant figures to display
   * @param size --- maximum number of charaters in the display
   **/
  public  DoubleGraphicsDisplay(double v,int sigfig,int size){
    abs_size=Math.max(sigfig+3,size);
    sig=sigfig;
    // leave off the rest until we have a font
    llCorner=new Point(0,0);
    setValue(v);
  }

  public DoubleGraphicsDisplay(double v,int sigfig){
    this(v,sigfig,0);
  }
  
  public DoubleGraphicsDisplay(int sigfig){
    this(0.0,sigfig,0);
  }
  public DoubleGraphicsDisplay(int sigfig,int size){
    this(0.0,sigfig,size);
  }

  /**
   * @returns the number of significant figures in display
   **/
  public int getSigFigs(){
    return sig;
  }

  /**
   * establishes various internal variables, which are needed 
   * to draw the value
   **/
  public void measure(Font abs){
    Canvas tmp=new Canvas();
    abs_fn=abs;
    abs_fm=tmp.getFontMetrics(abs_fn);
    exp_fn=new Font( abs_fn.getName(), //same style
		     abs_fn.getStyle(),
		     3*abs_fn.getSize()/4 // but smaller
		     );
    exp_fm=tmp.getFontMetrics(exp_fn);
    exp_y=-abs_fm.getHeight()+exp_fm.getHeight();
  }

  public int getHeight(){
    return exp_y+exp_fm.getHeight();
  }
  public int getWidth(){
    return (abs_size+1)*abs_fm.charWidth('0')+4*exp_fm.charWidth('0');
  }
  public Dimension getExtent(){
    return new Dimension(getWidth(),getHeight());
  }

  public void draw(Graphics g){
    if( abs_fn==null) measure( g.getFont() );
    
    g.drawString( abs, llCorner.x,llCorner.y);
    if( !EMPTY.equals(exp) ){
      exp_x=abs_fm.stringWidth(abs)+1; // x position of exponent
                                       // relative to the llCorner
      g.drawString(exp,llCorner.x+exp_x,llCorner.y+exp_y);
    }
  }


  /**
   * sets the value to display
   **/
  public void setValue(double v){
    value=v;
    
    int sign;
    if( v < 0.0 ){
      sign=-1;
      v=-v;
    }
    else{
      sign=+1;
    }
    int pow;
    if( v > 0.0 ){
      pow= (int)( log10(v) );
      if( v<1.0 ) --pow;
    }
    else pow=0;
    
    if(  pow > sig-2 || pow < -sig+2 ){
      setUpExp(sign,v,pow);
    }
    else setUpFP(sign,v,pow);
  }

  public double getValue(){ return value; }


  /**
   * sets the position of the lower left corner.
   * used to move the display around
   **/
  public void setLLCorner(int x,int y){
    llCorner.x=x; llCorner.y=y;
  }
  public void setLLCorner(Point q){
    setLLCorner(q.x,q.y);
  }

  public Point getLLCorner(){
    return llCorner;
  }



  void setUpFP(int sign,double value,int pow){
    if( value < 1.0 ) ++pow;
    double factor=ipow(10.0,pow+sig);
    double new_value=( Math.round( value*factor) )/factor;
    
    StringBuffer tmp;
    if( sign < 0 ) tmp=new StringBuffer("-");
    else           tmp=new StringBuffer(" ");
    tmp.append(new_value);
    
    if( tmp.toString().indexOf('E') > -1 || 
	tmp.toString().indexOf('.') > abs_size-1){
      setUpExp(sign,value,pow);
      return;
    }

    while( tmp.length() < abs_size ) tmp.append('0');
    
    tmp.setLength( abs_size );
    abs=tmp.toString();
    exp=EMPTY;
  }
  
  void setUpExp(int sign,double value,int pow){
    value= ( Math.round( value*ipow(10.0, sig-pow-1) ))/ipow(10.0,sig-1);
    
    
    StringBuffer tmp;
    if( sign < 0 ) tmp=new StringBuffer("-");
    else           tmp=new StringBuffer(" ");
    
    tmp.append(value);
    while( tmp.length() < sig+2 ) tmp.append('0');
    tmp.setLength( sig+2 );
    tmp.append("x10");
    abs=tmp.toString();
    exp=String.valueOf(pow);
  }
  
  
  
  

  static final double LN10=Math.log(10.0);
  
  
  final double log10(double v){
    return Math.log(v)/LN10;
  }
  final double ipow(double base,int exp){
    if( exp < 0 ) return 1.0/ipow(base,-exp);
    if( exp==0 ) return 1.0;
    if( exp==1 ) return base;
    return ipow( base*base, exp/2 )*ipow(base,exp%2);
  }
}


