package utils.GUI;

import java.awt.*;
import utils.GUI.DoubleTextDisplay;
import utils.GUI.GridBagHelpers;



/**********************************************************
 * a component which has: a Label, a Scrollbar and a DoubleTextDisplay.
 * the value depends linearly on the position of the Scrollbar marker.
 *
 * when the user changes the value of the component by moving the
 * Srollbar, it generates ACTION_EVENTs with an arg that is 
 * a Double containing the new value of the component
 *************************************************************/

public class LabelledScroller extends Panel{
  protected double offset;
  protected double stepsize;
  protected Label label;
  protected DoubleTextDisplay disp;
  protected Scrollbar      bar;

  
  /**
   * @param l == label
   * @param sigfig == number of significant figures
   * @param size  == maximum number of characters in the display
   **/
public LabelledScroller(String l,int sigfig,int size){
  super();
  setLayout(new GridBagLayout());
  label=new Label(l);
  disp=new DoubleTextDisplay(sigfig,size);
  bar=new Scrollbar(Scrollbar.HORIZONTAL,
		    0,1,0,100 );
  offset=0.0;
  stepsize=1.0;
  fixField();
  GridBagHelpers.insert(this,label,
			1,1,
			1,1,
			0,5,0,0,
			GridBagConstraints.CENTER
			);
  GridBagHelpers.insert(this,disp,
			2,1,
			1,1,
			0,0,3,5,
			GridBagConstraints.EAST
			);
  GridBagHelpers.insert_fill(this,bar,
			1,2,
			2,1,
			3,5,0,5
			);
}

  /**
   * just specify the label
   **/
public LabelledScroller(String l){
  this(l,3,6);
}
  /**
   * specify the label, and the range of allowed values
   **/
public LabelledScroller(String l,double min,double max){
  this(l);
  setRange(min,max);
}
public void addNotify(){
  super.addNotify();
  disp.setBackground( getBackground().brighter());
}
public boolean handleEvent(Event e){
  if(e.id==Event.SCROLL_LINE_UP ||
     e.id==Event.SCROLL_LINE_DOWN   ||
     e.id==Event.SCROLL_PAGE_UP ||
     e.id==Event.SCROLL_PAGE_DOWN ||
     e.id==Event.SCROLL_ABSOLUTE 
     ){
    fixField();
                          //  prepare the action to be sent up
    e.target=this;
    e.id=Event.ACTION_EVENT;
    e.arg= (new Double(getValue()));
    return false;
  }
  else{
    return super.handleEvent(e);
  }
}


  /**
   * set the range of values the user is able to scroll through
   **/
public void setRange(double min,double max){
  offset=min;
  stepsize=(max-min)/100.0;
  fixField();
}

public double getValue(){
  return (offset+ (double)(stepsize*bar.getValue()) );
}

public void setValue(double d){
  bar.setValue( (int)( (d-offset)/stepsize ) );
  fixField();
}
public void fixField(){
  disp.setValue( getValue() );
}

  public double getMinValue(){ return offset; }
  public double getMaxValue(){ return offset+100*stepsize; }


}// end of LabelledScroller











