package utils.GUI.d3;

import java.awt.*;
import utils.GUI.EventIDS;
import utils.GUI.GridBagHelpers;
import utils.d3.*;


class PerspButton extends Canvas{


  // these constants are chosen so that they correspond
  // to keys on the numeric keypad
  public static final int RIGHT=6;
  public static final int DOWN=2;
  public static final int LEFT=4;
  public static final int UP=8;
  static final int MIN=16;
  static final int PREF=20;
  static final int BORDER=3;

  Polygon picture;
  int type;
  boolean up;

  public PerspButton(int ty){
    type=ty;
    //    picture=new Polygon();
    up=true;
    //    setUpPicture(ty);
  }

  public void resize(int w,int h){
    super.resize(w,h);
    setUpPicture(type);
  }

  public Dimension minimumSize(){ return new Dimension(MIN,MIN); }
  public Dimension preferredSize(){ return new Dimension(PREF,PREF); }

  public void update(Graphics g){
    paint(g);
  }
  public void paint(Graphics g){
    Dimension d=size();
    if( picture==null) setUpPicture(type);
    g.setColor( getBackground());
    g.fillRect( 0,0,d.width,d.height);
    for(int i=0;i<BORDER;++i){
      g.draw3DRect(i,i,d.width-1-2*i,d.height-1-2*i,up);
    }
    g.setColor(getForeground());
    g.fillPolygon(picture);
  }

  public boolean mouseDown(Event e,int x,int y){
    // make it look like a key-event
    up=false;
    repaint();
    postEvent( new Event(this,0,Event.KEY_PRESS,x,y,type,e.modifiers));
    return true;
  }
  public boolean mouseUp(Event e,int x,int y){
    e.key=type;
    up=true;
    repaint();
    postEvent(new Event(this,0,Event.KEY_RELEASE,x,y,type,e.modifiers));
    return true;
  }


  void setUpPicture(int ty){
    Dimension mysize=size();
    picture=new Polygon();
    switch(ty){
    case RIGHT: setRight(mysize);   break;
    case DOWN:  setDown(mysize);    break;
    case LEFT:  setLeft(mysize);    break;
    case UP:    setUp(mysize);      break;
    default:
      throw new 
	IllegalArgumentException("illegal arg to setup in PerspButton");
    }
    repaint();
  }

  void setRight(Dimension mysize){
    int xmin=mysize.width/4;
    int ymin=mysize.height/4;
    int xmax=3*xmin;
    int ymax=3*ymin;
    picture.addPoint( xmax, 2*ymin );
    picture.addPoint( xmin, ymax );
    picture.addPoint( xmin, ymin );
  }
  void setDown(Dimension mysize){
    int xmin=mysize.width/4;
    int ymin=mysize.height/4;
    int xmax=3*xmin;
    int ymax=3*ymin;
    picture.addPoint( 2*xmin, ymax );
    picture.addPoint( xmin, ymin );
    picture.addPoint( xmax, ymin );
  }
  void setLeft(Dimension mysize){
    int xmin=mysize.width/4;
    int ymin=mysize.height/4;
    int xmax=3*xmin;
    int ymax=3*ymin;
    picture.addPoint( xmin, 2*ymin );
    picture.addPoint( xmax, ymin );
    picture.addPoint( xmax, ymax );
  }
  void setUp(Dimension mysize){
    int xmin=mysize.width/4;
    int ymin=mysize.height/4;
    int xmax=3*xmin;
    int ymax=3*ymin;
    picture.addPoint( 2*xmin, ymin );
    picture.addPoint( xmax, ymax );
    picture.addPoint( xmin, ymax );
  }
    
}

public class OriginWatcherControl 
extends Panel
implements Runnable{

  public static final String ZOOMIN= "Zoom In ";
  public static final String ZOOMOUT="Zoom Out";
  public static  double MAGFACTOR=1.2;

  Perspective p;
  RotMat      scm;
  ThreeVector scv;
  boolean going;
  double dth;
  
  
  
  public OriginWatcherControl(Perspective per){
    super();
    p=per;
    scv=new ThreeVector();
    scm=new RotMat();
    going=false;
    dth=p.getViewAngle()/6.0;
    myLayout();
  }



  public boolean keyDown(Event e,int key){
    if(!going){
      switch(key){
      case PerspButton.RIGHT: 
	start_horizontal(1.0);
	return true;
      case PerspButton.LEFT:
	start_horizontal(-1.0);
	return true;
      case PerspButton.DOWN:
	start_vertical(1.0);
	return true;
      case PerspButton.UP:
	start_vertical(-1.0);
	return true;
      default:
	return true;
      }
    }
    return true;
  }
  
  public boolean keyUp(Event e,int key){
    going=false;
    return true;
  }

  public boolean action(Event e,Object arg){
    if( ZOOMIN.equals(arg)){
      p.magnify( MAGFACTOR );
      postEvent( new Event(this,EventIDS.PERSPECTIVE_EVENT,null) );
      return true;
    }
    else if( ZOOMOUT.equals(arg)){
      p.magnify( 1.0/MAGFACTOR);
      postEvent( new Event(this,EventIDS.PERSPECTIVE_EVENT,null) );
      return true;
    }
    else return false;
  }

  void start_horizontal(double way){
    scv.set_cro(p.getLocation(),p.getRight());
    scm.set_rotationAbout(scv,way*dth);
    Thread tmp=new Thread(this);
    going=true;
    tmp.start();
  }
  void start_vertical(double way){
    scv.set_cro(p.getLocation(),p.getDown());
    scm.set_rotationAbout(scv,way*dth);
    Thread tmp=new Thread(this);
    going=true;
    tmp.start();
  }




  public void run(){
    while(going){
      p.crane(scm);
      p.setLocation( scm.actingOn(p.getLocation()));
      postEvent( new Event(this,EventIDS.PERSPECTIVE_EVENT,null) );
      try{
	Thread.currentThread().sleep(100);
      }
      catch(InterruptedException ie){}
    }
  }

  void myLayout(){
    setLayout(new BorderLayout());
    Panel pan=new Panel();
    pan=makeControls();
    add("North",pan);
    pan=new Panel();
    pan.add(new Button(ZOOMIN));
    add("Center",pan);
    pan=new Panel();
    pan.add( new Button(ZOOMOUT));
    add("South",pan);
  }

  Panel makeControls(){
    Panel tmp=new Panel();
    tmp.setLayout(new GridBagLayout());
    GridBagHelpers.insert( tmp,
				new PerspButton(PerspButton.UP),
				2,1,
				1,1);
    GridBagHelpers.insert( tmp,
				new PerspButton(PerspButton.RIGHT),
				3,2,
				1,1);
    GridBagHelpers.insert( tmp,
				new PerspButton(PerspButton.DOWN),
				2,3,
				1,1);
    GridBagHelpers.insert( tmp,
				new PerspButton(PerspButton.LEFT),
				1,2,
				1,1);
    return tmp;
  }

}











