package utils.IO;

import utils.IO.FileFormatEx;
import java.io.InputStream;
import java.io.IOException;
import java.io.StreamTokenizer;

/** 
   extension of StreamTokenizer which tries to format the 
	next next token in the specified way.
**/


public class StreamReader extends StreamTokenizer{
  InputStream source;
  
public StreamReader(InputStream in){
  super(in);
  source=in;
}

public double readDouble() throws IOException,FileFormatEx{
  nextToken();
  
  if( ttype!=TT_NUMBER && ttype!='"' ){
    throw new FileFormatEx("expecting a double, read in : "+sval );
  }
  if( ttype==TT_NUMBER) return nval;
  else{
    return Double.valueOf(sval).doubleValue();
  }
}

public float readFloat() throws IOException,FileFormatEx{
  nextToken();
  
  if( ttype!=TT_NUMBER && ttype!='"' ){
    throw new FileFormatEx("expecting a float read in : "+sval);
  }
  if( ttype==TT_NUMBER ) return (float)nval;
  else{
    return Float.valueOf(sval).floatValue();
  }
}

public long readLong() throws IOException,FileFormatEx{
  nextToken();
  
  if( ttype!=TT_NUMBER && ttype!='"' ){
    throw new FileFormatEx("expecting a long read in : "+sval);
  }
  if( ttype==TT_NUMBER ) return (long)nval;
  else{
    return Long.valueOf(sval).longValue();
  }
}


public int readInt() throws IOException,FileFormatEx{
  nextToken();
  
  if( ttype!=TT_NUMBER && ttype!='"' ){
    throw new FileFormatEx("expecting an int read in : "+sval);
  }
  if( ttype==TT_NUMBER ) return (int)nval;
  else{
    return Integer.valueOf(sval).intValue();
  }
}

public String readString() throws IOException,FileFormatEx{
  nextToken();
  if( ttype!=TT_WORD && ttype!='"'){
    throw new FileFormatEx("expecting a string read in: "+sval);
  }
  return sval;
}

public void skipToWord() throws IOException{
  while( nextToken()!= TT_WORD && ttype!='"' && ttype!=TT_EOF){
	/* do nothing */
  }
}
/** skips to the next word, checks to make sure that the word
	found matches the given string **/
public void skipToWord(String match_pattern) throws IOException,FileFormatEx{
  skipToWord();
  if( ! match_pattern.equals(sval) ){
    throw new FileFormatEx(
	     "looking for pattern "+match_pattern+" found "+sval);
  }
}
public void close(){ 
  try{source.close();}
  catch(IOException ioe){
    System.out.println(ioe.getMessage());
  }
}
}


