package utils.d2;


/****************************************************************
 * implements a circular region 
 ****************************************************************/
public final class CircRegion implements Regional{
  
  TwoVector center;
  double    r;
  double    r2;
  
 
  CircRegion(){ center=new TwoVector(); }
  public CircRegion(TwoVector loc,double rad){
    this();
    setCenter(loc);
    setR(rad);
  }

  
  public boolean containsPoint(TwoVector a){
    TwoVector disp=a.minus(center);
    return ( disp.dot(disp) < r2 );
  }
  
  public boolean containsDisk(TwoVector a,double rad){
    TwoVector disp=a.minus(center);
    double dist=disp.norm();
    return ( dist < r-rad );
  }
 
  public double getMinX(){ return center.xComponent()-r; }
  public double getMaxX(){ return center.xComponent()+r; }
  public double getMinY(){ return center.yComponent()-r; }
  public double getMaxY(){ return center.yComponent()+r; }

  public void translate(TwoVector a){ center.plusEquals(a); }




  public TwoVector getCenter(){ return center; }
  public double getR(){ return r; }

  public void setCenter(TwoVector a){
    center.setEqual(a);
  }
  public void setR(double new_rad){
    if( new_rad < 0.0 ) 
      throw new IllegalArgumentException(
        "CircRegion: negative radius specified" );
    
    r=new_rad;
    r2=new_rad*new_rad;
  }

  public boolean overlaps(CircRegion that){
    TwoVector disp=this.center.minus( that.center );
    double dist=disp.norm();
    return ( dist< this.r + that.r );
  }

  // in principle could put in static functions analogous to
  // those in RectRegion
}
			     




