package utils.d2;

/*************************************************************************
 * a class which implements Regional as a rectangular box, with
 * sides parallel to the X and Y axes.  In additon to the Regional interface
 * it provides static methods which operate on the bounding RectRegions
 * of other Regional classes
 ***********************************************************************/

public class RectRegion implements Regional{
  
  TwoVector UL;
  TwoVector LR;
  
  
  RectRegion(){
    UL=new TwoVector();
    LR=new TwoVector();
  }

  /**
   * construct by specifying the corners
   **/
  public RectRegion(TwoVector a,TwoVector b){
    this();
    setCorners(a,b);
  }
  public RectRegion(RectRegion b){
    this( b.getUL(), b.getLR() );
  }



  //-------------------------------------------------------
  // Regional interface 

  public boolean containsDisk(TwoVector a,double r){
    return (
	    ( a.xComponent() < getMaxX()-r )&&
	    ( a.xComponent() > getMinX()+r )&&
	    ( a.yComponent() < getMaxY()-r )&&
	    ( a.yComponent() > getMinY()+r )
	    );
  }
  public boolean containsPoint(TwoVector a){
    return (
	    ( a.xComponent() < getMaxX() )&&
	    ( a.xComponent() > getMinX() )&&
	    ( a.yComponent() < getMaxY() )&&
	    ( a.yComponent() > getMinY() )
	    );
  }
  
  public double getMinX(){ return UL.xComponent(); }
  public double getMaxX(){ return LR.xComponent(); }
  public double getMinY(){ return LR.yComponent(); }
  public double getMaxY(){ return UL.yComponent(); }
  
  public void translate(TwoVector d){
    UL.plusEquals(d);
    LR.plusEquals(d);
  }

  //----------------------------------------------------
  // specific functions

  final void set(double x_min,double x_max,
		 double y_min,double y_max ){
    UL.set_xy( x_min, y_max );
    LR.set_xy( x_max, y_min );
  }

  /**
   * re-sets the corners of this RectRegion
   **/
  public final void setCorners(TwoVector a,TwoVector b){
    double xn=Math.min( a.xComponent(),b.xComponent());
    double xx=Math.max( a.xComponent(),b.xComponent());
    double yn=Math.min( a.yComponent(),b.yComponent());
    double yx=Math.max( a.yComponent(),b.yComponent());

    set(xn,xx,yn,yx);
  }


  public double getExtentX(){ return getMaxX()-getMinX(); }
  public double getExtentY(){ return getMaxY()-getMinY(); }

  /**
   * returns the upper left corner ( min x, max y )
   **/
  public TwoVector getUL(){ return new TwoVector(UL); }
  
  /**
   * returns the lower right corner ( max x, min y )
   **/
  public TwoVector getLR(){ return new TwoVector(LR); }
  
  /**
   * returns the upper right corner ( max x, max y )
   **/
  public TwoVector getUR(){
    return new TwoVector( getMaxX(), getMaxY() );
  }
  
  /**
   * returns the lower left corner ( min x, min y)
   **/
  public TwoVector getLL(){
    return new TwoVector( getMinX(), getMinY() );
  }

  /**
   * returns true if this and that share at least one point
   **/
  public boolean overlaps(RectRegion that){
    return overlap(this,that);
  }

  //---------------------------------------------------------
  // static functions to operate on other regions, by in effect
  // casting them as RectRegions

  /**
   * in effect a casting operation
   **/
  public static RectRegion boundingBox(Regional shape){
    RectRegion tmp=new RectRegion();
    tmp.set( shape.getMinX(), shape.getMaxX(),
	     shape.getMinY(), shape.getMaxY() );
    return tmp;
  }

  /**
   * determines if the <em> bounding boxes </em> overlap 
   **/
  public static boolean overlap(Regional a,Regional b){
    return (
	    ( a.getMinX() < b.getMaxX() )&&
	    ( b.getMaxX() > b.getMinX() )&&
	    ( a.getMinY() < b.getMaxY() )&&
	    ( b.getMaxY() > b.getMinY() )
	    );
  }
  
  /**
   * returns the rectangular intersection of the <em> bounding boxes </em>
   **/
  public static RectRegion intersection(Regional a,Regional b){
    
    if( overlap(a,b) ){
      RectRegion tmp=new RectRegion();
      double xn=Math.max( a.getMinX(), b.getMinX() );
      double xx=Math.min( a.getMaxX(), b.getMaxX() );
      double yn=Math.max( a.getMinY(), b.getMinY() );
      double yx=Math.min( a.getMaxY(), b.getMaxY() );
      tmp.set( xn, xx,
	       yn, yx );
      return tmp;
    }
    else{
      return null; 
    }
  }
  
  /**
   * returns the rectangular union of the <em> bounding boxes </em> 
   **/
  public static RectRegion union(Regional a,Regional b){
    RectRegion tmp=new RectRegion();
    double xn=Math.min( a.getMinX(), b.getMinX() );
    double xx=Math.max( a.getMaxX(), b.getMaxX() );
    double yn=Math.min( a.getMinY(), b.getMinY() );
    double yx=Math.max( a.getMaxY(), b.getMaxY() );
    tmp.set( xn, xx,
	     yn, yx );
    return tmp;
  }
  
}





























