package utils.d2;

/********************************************************************
 * interface for objects which define regions in real-2D-space.
 * in many cases consider a RectRegion to be fundamental.
 ********************************************************************/

public interface Regional{
  
  /**
   * returns true if the given location is inside the region.
   **/
  public boolean containsPoint(TwoVector a);
  /**
   * returns true if a disk of the given radius at the given location is
   * completely inside the region
   **/
  public boolean containsDisk(TwoVector a,double r);
  
  /**
   * all points in the region have x component greater than x min
   **/
  public double getMinX();
  /**
   * all points in the region have x component less than x max
   **/
  public double getMaxX();
  /**
   * all points in the region have y component greater that y min
   **/
  public double getMinY();
  /**
   * all points in the region have y component less than y max
   **/
  public double getMaxY();
 
  /**
   * move the region
   **/
  public void translate(TwoVector d);
}



