/**
handles transformations between real-space (i.e. specified by TwoVectors)
coordinates and display cooridinates
**/
package utils.d2;

import utils.d2.TwoVector;
import java.awt.Point;
import java.awt.Dimension;

public final class TwoPerspective{
  Point center;
  Point origin;
  TwoVector centermapsto;
  double rscale;
  double scale;
  boolean changed;         // here for convienience  ------
                           // user is responsible for clearing the change
/** default.
makes a TwoPerspective where the origin(real space) maps to the point (0,0)
(screen coordinates). and sets the scale to 1.0;
**/
public TwoPerspective(){
  center=new Point(0,0);
  origin=new Point(0,0);
  centermapsto=new TwoVector();
  scale=1.0;
  rscale=1.0;
  changed=true;
}
/** recommended.
looks at the origin. is positioned so that the origin is at the
center of the given dimension; scale is chosen so that realscale
pretty much fills up the given Dimension
**/
public TwoPerspective(Dimension viewsize,double realscale){
  this();
  rscale=realscale;
  resetScales(viewsize,realscale);
  changed=true;
}
/**
moves the observer in real space
**/
public void translate(TwoVector a){
  centermapsto.plusEquals(a);
  fixOrigin();
  changed=true;
}

public void magnify(double factor){
  scale*=factor;
  fixOrigin();
  changed=true;
}
/** recenters the perspective. 
i.e. once this is called the origin will map to the center of the given
Dimension
**/
public void resize(Dimension d){
  resetScales(d,rscale);
  changed=true;
}
  public void resize(int width,int height){
    resetScales( new Dimension(width,height),rscale);
    changed=true;
  }
/** rescales and centers the perspective.
**/
public void resetScales(Dimension viewsize,double realscale){
  center=new Point(viewsize.width/2,viewsize.height/2);
  rscale=realscale;
  scale=((double)(center.x+center.y))/realscale;
  fixOrigin();
  changed=true;
}

/** 
@retruns the Point on the display which corresponds to a
**/
public final Point getDisplay(TwoVector a){
  Point x=new Point(0,0);
  getDisplay(x,a);
  return x;
}
/** 
@sets the values in the  Point on the display which corresponds to a
**/
public void getDisplay(Point p,TwoVector a){
  getDisplacement(p,a);
  p.translate( origin.x, origin.y );
}

  public final Point getDisplacement(TwoVector a){
    Point x=new Point(0,0);
    getDisplacement(x,a);
    return x;
  }
  public void getDisplacement(Point p,TwoVector a){
    p.x=(int)Math.round(a.x*scale);
    p.y=-(int)Math.round(a.y*scale);
  }

/** 
returns the TwoVector which maps to the given on screen Point 
**/
  public TwoVector getRealPosition(Point p){
    return getRealPosition(p.x,p.y);
  }

  public TwoVector getRealPosition(int d_x,int d_y){
    double x=(double)(d_x-center.x)/scale+centermapsto.x;
    double y=(double)(center.y-d_y)/scale+centermapsto.y;
    return new TwoVector(x,y);
  }

  public TwoVector getRealDisplacement(int dx,int dy){
    double x=(double)(dx)/scale;
    double y=(double)(-dy)/scale;
    return new TwoVector(x,y);
  }
/**
@returns the on screen location of the center
**/
public Point getCenter(){ return center;}
/**
@returns the coordinates(real) which the center corresponds to
**/
public TwoVector getCenterMapsTo(){ return centermapsto; }
public double getScale(){ return scale; }
/**
@returns whether or not the perspective has changed -- useful in determining
whether or not an object needs to adjust how it is displayed
**/
public boolean hasChanged(){ return changed;}
public void   clearChanged(){
  changed=false;
}
public void setChanged(){ changed=true; }


  void fixOrigin(){
    origin=getDisplacement(centermapsto.times(-1.0));
    origin.translate( center.x, center.y );
  }

}// end of TwoPerspective











