package utils.d2.primitive;

import utils.d2.*;

import utils.d2.primitive.Locatable;
import java.awt.Point;
import java.util.StringTokenizer;

/**********************************************************************
 * class for drawing disks associated with a Locatable.
 * the "F" indicates that it does not depend on the orientation of the 
 * Locatable; i.e. it will always be drawn with the given offset.
 **********************************************************************/
public class FDisk implements utils.d2.primitive.SingleGP{
  public static final String TYPE=KnownGP.FDISK;

  Locatable master;
  TwoVector r_offset;   // real space variables
  double    r_rad;      // 

  Point     d_offset;   // display variables
  int       d_rad;
  int       d_dia;
  
  boolean needs_update;
  boolean hittable;

  public FDisk(Locatable owner,TwoVector offset,double r){
    master=owner;
    r_offset=new TwoVector(offset);
    r_rad=r;
    
    d_offset=new Point(0,0);
    needs_update=true;
    hittable=true;
  }


  public void allowHits(boolean val){
    hittable=val;
  }
  public boolean canBeHit(){ return hittable; }
  
  public void updateDisplay(TwoPerspective p){
    if( needs_update || p.hasChanged() ){
      p.getDisplacement( d_offset,r_offset );
      d_rad=(int)(Math.round( p.getScale()*r_rad ));
      d_rad=Math.max(1,d_rad);  // make sure that it will show up on screen
      d_dia=2*d_rad;
      needs_update=false;
    }
  }

  public void draw(java.awt.Graphics g){
    Point q=master.getDisplay();
    g.fillOval( q.x+d_offset.x-d_rad, q.y+d_offset.y-d_rad, d_dia, d_dia );
  }

  public boolean isHitBy(int x,int y){
    if(hittable){

      Point q=master.getDisplay();
      int dx=x-(q.x+d_offset.x);
      int dy=y-(q.y+d_offset.y);
      
      dx*=dx;
      dy*=dy;
      
      if( dx+dy < d_rad*d_rad ) return true;
    }
    return false;
  }

  public Object clone(){
    return new FDisk(master,r_offset,r_rad);
  }

  public TwoVector getOffset(){ return r_offset; }
  public double    getRadius(){ return r_rad; }
  
  public void setOffset(TwoVector off){
    r_offset=new TwoVector(off); needs_update=true;
  }
  public void setRadius(double r){ r_rad=r; needs_update=true; }


  public Locatable getMaster(){ return master; }
  public void      setMaster(Locatable l){ master=l; }


  /**
   * does not include data on the master
   **/
  public String toString(){
    String tmp=getType()+" ";
    tmp+=("radius "+getRadius());
    tmp+=( "x "+getOffset().xComponent()+" ");
    tmp+=( "y "+getOffset().yComponent()+" ");
    return tmp;
  }
  
  public String getType(){ return TYPE; }
    
}














