package utils.d2.primitive;

import utils.d2.*;

import java.awt.Point;
import java.awt.Polygon;
import java.awt.Graphics;

import java.util.StringTokenizer;

public class FPolygon extends Polygon implements SingleGP{

  Locatable master;
  
  TwoVector[] r_offset;
  int[]       d_x;
  int[]       d_y;
  
  int updated;
  int max;

  static final int BLOCK=6;
  Point scratch;
  boolean hittable;
  
  public FPolygon(Locatable owner){
    master=owner;
    r_offset=new TwoVector[BLOCK];
    d_x=new int[BLOCK];
    d_y=new int[BLOCK];
    scratch=new Point(0,0);
    max=BLOCK;
    updated=0;
    hittable=true;
  }

  public FPolygon(Locatable owner,TwoVector[] offs){
    this(owner);
    augmentCapacity( offs.length );
    for(int i=0;i<offs.length;++i){
      addVertex(offs[i]);
    }
  }
  
  public boolean canBeHit(){ return hittable; }
  public void allowHits(boolean val){ hittable=val; }


  public synchronized void addVertex(TwoVector location){
    augmentCapacity(1);
    
    r_offset[npoints]=new TwoVector(location);
    addPoint(0,0);
  }
  
  public void updateDisplay(TwoPerspective p){
    if( p.hasChanged() ) updated=0;
    for( int i=updated; i<npoints;++i){
      p.getDisplacement(scratch, r_offset[i] );
      d_x[i]=scratch.x;
      d_y[i]=scratch.y;
    }
    updated=npoints;
  }
  
  public void draw(Graphics g){
    Point q=master.getDisplay();

    for(int i=0;i<npoints;++i){      
      xpoints[i]=q.x+d_x[i];
      ypoints[i]=q.y+d_y[i];
    }
    g.fillPolygon( this );
  }


  public boolean isHitBy(int x,int y){ return hittable&&inside(x,y); }

  public Object clone(){
    return new FPolygon(master,r_offset);
  }

  public TwoVector getVertex(int i){
    return r_offset[i];
  }
  public void setVertex(int i,TwoVector new_location){
    r_offset[i]=new TwoVector(new_location);
    updated=Math.min( updated, i);
  }


  public void setMaster(Locatable owner){ master=owner; }
  public Locatable getMaster(){ return master; }

  void augmentCapacity(int i){
    if( npoints+i>max ){
      max+=(BLOCK+i);
      TwoVector[] oldv=r_offset;
      int[]       oldx=d_x;
      int[]       oldy=d_y;
      
      r_offset=new TwoVector[max];
      d_x=new int[max];
      d_y=new int[max];
      
      System.arraycopy(oldv,0,r_offset,0,npoints);
      System.arraycopy(oldx,0,d_x,0,npoints);
      System.arraycopy(oldy,0,d_y,0,npoints);
    }
  }

  public String getType(){ return KnownGP.FPOLYGON; }
  
  public String toString(){
    String tmp="points "+npoints+" ";
    
    for(int i=0;i<npoints;++i){
      tmp+=( "x "+getVertex(i).xComponent()+" y "+getVertex(i).yComponent()
	     +" ");
    }
    return tmp;
  }
  
}




