package utils.d2.primitive;

import utils.d2.*;
import java.awt.Point;
import java.awt.Polygon;

public class FThinArrow implements SingleGP{
  public static final double C=Math.cos(Math.PI/6.0);
  public static final double S=Math.cos(Math.PI/6.0);
  public static final int HEADLENGTH=7;

  Locatable master;
  TwoVector offset;
  TwoVector value;
  boolean needs_update;
  Point d_off;
  Point d_val;
  Polygon tip;

  public FThinArrow(Locatable owner,TwoVector off,TwoVector val){
    master=owner;
    offset=new TwoVector(off);
    value=new TwoVector(val);
    needs_update=true;
    d_off=new Point(0,0);
    d_val=new Point(0,0);
    tip=new Polygon(new int[3],new int[3],3);
  }
  
  public void updateDisplay(TwoPerspective p){
    if( needs_update || p.hasChanged()){
      p.getDisplacement(d_off,offset);
      p.getDisplacement(d_val,value);
      
      double a,o,h;
      a=(double)(d_val.y);
      o=(double)(d_val.x);
      h=Math.sqrt( a*a+o*o);
      a/=h;
      o/=h;
      
      d_val.translate( d_off.x,d_off.y);

      tip.xpoints[0]=d_val.x-(int)(HEADLENGTH*(C*o-S*a));
      tip.ypoints[0]=d_val.y-(int)(HEADLENGTH*(C*a+S*o));
      
      tip.xpoints[1]=d_val.x-(int)(HEADLENGTH*(C*o+S*a));
      tip.ypoints[1]=d_val.y-(int)(HEADLENGTH*(C*a-S*o));
      
      tip.xpoints[2]=d_val.x;
      tip.ypoints[2]=d_val.y;
      
      needs_update=false;
    }
  }
  
  public void draw(java.awt.Graphics g){
    Point q=master.getDisplay();
    //    q.translate( d_off.x,d_off.y);
    g.drawLine( d_off.x+q.x , d_off.y+q.y , 
		q.x+d_val.x , q.y+d_val.y );
    for(int i=0;i<3;++i){
      tip.xpoints[i]+=q.x;
      tip.ypoints[i]+=q.y;
    }
    g.fillPolygon(tip);
  }

  public Object clone(){
    return new FThinArrow(master,offset,value);
  }
  public String getType(){ return KnownGP.FTHINARROW; }

  public boolean canBeHit(){ return false; }
  public boolean isHitBy(int x,int y){ return false; }
  
  public void setMaster(Locatable owner){
    master=owner;
  }
  public Locatable getMaster(){ return master; }
  
  public void setOffset(TwoVector new_off){
    offset.setEqual( new_off );
    needs_update=true;
  }
  public TwoVector getOffset(){ return offset; }
  
  public void setValue(TwoVector new_val){
    value.setEqual(new_val);
    needs_update=true;
  }
  public TwoVector getValue(){ return value; }


  public String toString(){
    StringBuffer tmp=new StringBuffer();
    tmp.append( getType()).append(" ");
    tmp.append("offset_x ").append( offset.xComponent()).append(" ");
    tmp.append("offset_y ").append( offset.yComponent()).append(" ");
    tmp.append("value_x ").append( value.xComponent()).append(" ");
    tmp.append("value_y ").append( value.yComponent());
    return tmp.toString();
  }

}
