package utils.d2.primitive;

import utils.d2.*;

import utils.d2.primitive.Locatable;
import java.awt.Point;

import java.util.StringTokenizer;

/********************************************************************
 *  a class where the position of the disk depends on the 
 *  orientation of the locatable it is attached to.
 *  
 ********************************************************************/

public class RDisk extends utils.d2.primitive.FDisk{
  public static final String TYPE=KnownGP.RDISK;
  TwoVector fiduciary;
  
  public RDisk(Locatable owner,TwoVector offset,double r){
    super(owner,offset,r);
    fiduciary=new TwoVector(offset);
  }

  public void updateDisplay(TwoPerspective p){
    orient( master.getOrientation() );
    super.updateDisplay(p);
  }
  
  public Object clone(){
    return new RDisk(master,fiduciary,r_rad);
  }

  /** ----------------------------------------------------------
   *  sets the value of the <em>fiduciary</em> offset.
   *  the fiduciary offset is that when the Locatable has orientation=0.0
   **/
  public void setOffset(TwoVector off){
    fiduciary=new TwoVector(off); orient(master.getOrientation());
  }
  
  /**-----------------------------------------------------
   * returns the fiduciary offset.
   * i.e. the offset when the Locatable is in orientation 0.0
   **/
  public TwoVector getOffset(){ return fiduciary; }


  final void orient(double angle){
    // apply the rotation matrix from angle to the fiduciary vector
    // to obtain what the offset is now
    
    double c=Math.cos(angle);
    double s=Math.sin(angle);

    r_offset.set_xy(
		    c*fiduciary.xComponent()-s*fiduciary.yComponent(),
		    s*fiduciary.xComponent()+c*fiduciary.yComponent()
		    );
    needs_update=true;
  }


  
  public String getType(){ return TYPE; }

}









