package utils.d2.primitive;



import utils.d2.TwoVector;

import java.util.StringTokenizer;


public class SingleGPFactory implements KnownGP{
  



  public static SingleGP make(String s){
    StringTokenizer st=new StringTokenizer(s);
    
    String type=st.nextToken();
    
    if( FDISK.equals(type)) return makeFDisk(st);
    else if( RDISK.equals(type)) return makeRDisk(st);
    else if( FCIRCLE.equals(type)) return makeFCircle(st);
    else if( FTHINARROW.equals(type)) return makeFThinArrow(st);
    else if( FPOLYGON.equals(type)) return makeFPolygon(st);
    else if( RPOLYGON.equals(type)) return makeRPolygon(st);
    else if( FSEGMENT.equals(type)) return makeFSegment(st);
    else if( FPLINE.equals(type)) return makeFPLine(st);
    else if( RPLINE.equals(type)) return makeRPLine(st);
    else if( FSPECK.equals(type)) return makeFSpeck(st);
    else{
      System.out.println("Unkown SingleGP type: "+type);
      return null;
    }
  }
  
  

  static SingleGP makeFDisk(StringTokenizer st){
    skipLabel(st);
    double r=Double.valueOf(st.nextToken()).doubleValue();
    TwoVector off=readVector(st);
    
    return new FDisk(null,off,r);
  }
  static SingleGP makeRDisk(StringTokenizer st){
    skipLabel(st);
    double r=Double.valueOf(st.nextToken()).doubleValue();
    TwoVector off=readVector(st);
    
    return new RDisk(null,off,r);
  }
  
  static SingleGP makeFCircle(StringTokenizer st){
    skipLabel(st);
    double r=Double.valueOf(st.nextToken()).doubleValue();
    TwoVector off=readVector(st);
    
    return new FCircle(null,off,r);
  }
  static SingleGP makeFThinArrow(StringTokenizer st){
    TwoVector off=readVector(st);
    TwoVector val=readVector(st);
    
    return new FThinArrow(null,off,val);
  }

  static SingleGP makeFPolygon(StringTokenizer st){
    skipLabel(st);
    int n_points=Integer.valueOf(st.nextToken()).intValue();
    TwoVector[] points=new TwoVector[n_points];
    for(int i=0;i<n_points;++i){
      points[i]=readVector(st);
    }
    return new FPolygon(null,points);
  }
  static SingleGP makeRPolygon(StringTokenizer st){
    skipLabel(st);
    int n_points=Integer.valueOf(st.nextToken()).intValue();
    TwoVector[] points=new TwoVector[n_points];
    for(int i=0;i<n_points;++i){
      points[i]=readVector(st);
    }
    return new RPolygon(null,points);
  }
  static SingleGP makeFSegment(StringTokenizer st){
    TwoVector e1=readVector(st);
    TwoVector e2=readVector(st);
    
    return new FSegment(null,e1,e2);
  }
  static SingleGP makeFPLine(StringTokenizer st){
    skipLabel(st);
    int n_points=Integer.valueOf(st.nextToken()).intValue();
    TwoVector[] points=new TwoVector[n_points];
    for(int i=0;i<n_points;++i){
      points[i]=readVector(st);
    }
    return new FPLine(null,points);
  }
  static SingleGP makeRPLine(StringTokenizer st){
    skipLabel(st);
    int n_points=Integer.valueOf(st.nextToken()).intValue();
    TwoVector[] points=new TwoVector[n_points];
    for(int i=0;i<n_points;++i){
      points[i]=readVector(st);
    }
    return new RPLine(null,points);
  }

  static SingleGP makeFSpeck(StringTokenizer st){
    return new FSpeck(null);
  }
  protected static TwoVector readVector(StringTokenizer st){
    skipLabel(st);
    double x=Double.valueOf(st.nextToken()).doubleValue();
    skipLabel(st);
    double y=Double.valueOf(st.nextToken()).doubleValue();
    return TwoVector.new_xy(x,y);
  }
  
  
  protected static final void skipLabel(StringTokenizer st){ st.nextToken(); }
}







