package utils.d2.primitive;

import utils.d2.primitive.GraphicsPrimitive;
import utils.d2.TwoPerspective;
import java.awt.Graphics;

public class VectorOfPrimitives implements GraphicsPrimitive{
  public static final String TYPE="VectorOfPrimitives";

  GraphicsPrimitive[] stuff;
  int current;
  int max;
  int block;
  boolean canbehit;
  
  public VectorOfPrimitives(int initial,int bl){
    canbehit=false;
    max=Math.max(initial,2);
    current=0;
    stuff=new GraphicsPrimitive[max];
    block=Math.max(bl,2);
    
  }
  
  public VectorOfPrimitives(){
    this(10,10);
  }
  
  
  public void allowHits(boolean val){
    canbehit=val;
  }
  public boolean canBeHit(){ return canbehit; }

  public String getType(){ return TYPE; }

  public int add(GraphicsPrimitive gp){
    augmentCapacity(1);
    stuff[current]=gp;
    ++current;
    return current-1;
  }
  public synchronized void removeAll(){
    for(int i=current-1;i>=0;--i){
      removeAt(i);
    }
    current=0;
  }
  public int addSeperately(VectorOfPrimitives vop){
    augmentCapacity( vop.size() );
    System.arraycopy(vop.stuff,0,this.stuff,current,vop.size());
    current+=vop.size();
    return current-1;
  }
  
  void augmentCapacity(int increment){
    if( current+increment > max ){
      GraphicsPrimitive[] old=stuff;
      max=Math.max( max+block, current+increment );
      stuff=new GraphicsPrimitive[ max ];
      System.arraycopy(old,0,stuff,0,current);
    }
  }
  
  public GraphicsPrimitive primitiveAt(int i){
    return stuff[i];
  }
  public int size(){ return current; }
  
  public int indexOf( GraphicsPrimitive gp){
    for( int i=0;i<current;++i){
      if( stuff[i].equals(gp) ) return i;
    }
    return -1;
  }
  
  public GraphicsPrimitive removeAt(int i){
    GraphicsPrimitive tmp=stuff[i];
    int j=current-i-1;
    if( j>0 ){
      System.arraycopy( stuff,i+1,stuff,i,j);
    }
    --current;
    stuff[current]=null;
    return tmp;
  }
  
  public void updateDisplay(TwoPerspective p){
    for(int i=0;i<current;++i){
      stuff[i].updateDisplay(p);
    }
  }
  public void draw(Graphics g){
    for(int i=0;i<current;++i) stuff[i].draw(g);
  }

  GraphicsPrimitive findUnder(int x,int y){
    for(int i=0;i<current;++i){
      if( stuff[i].isHitBy(x,y) ) return stuff[i]; 
    }
    return null;
  }
  
  public boolean isHitBy(int x,int y){
    boolean tmp=canBeHit();
    int index=0;
    while( index < current &&   tmp){
      tmp|=stuff[index].isHitBy(x,y);
      ++index;
    }
    return tmp;
  }

  public int indexUnder(int x,int y){
    for(int i=0;i<current;++i){
      if( stuff[i].isHitBy(x,y) ) return i;
    }
    return -1;
  }



  public Object clone(){
    VectorOfPrimitives tmp=new VectorOfPrimitives(current,block);
    System.arraycopy(this.stuff,0,tmp.stuff,0,current);
    tmp.current=this.current;
    tmp.canbehit=this.canbehit;
    return tmp;
  }
}










