package utils.d3;


import java.awt.Dimension;





public class Perspective{
  /**
   * assumed distance (in pixels) between viewers eyes and the screen
   **/
  public static final double OBS=2000.0;

  public static final int RIGHT=1;
  public static final int DOWN=2;
  public static final int FORWARD=3;
  public static final int GENERAL=4;


  protected /*unit*/ ThreeVector forward;
  protected /*unit*/ ThreeVector down;
  protected /*unit*/ ThreeVector right;

  /**
   * the position of the observer 
   **/
  protected ThreeVector loc;
  
  /**
   * the tangent of the angle which defines the cone
   * of things which are visible
   **/
  protected double A;
  /**
   * the size of the view screen
   **/
  protected double size;

  /**
   * the center of the view
   **/
  int x,y;
  
  boolean changed;
   ThreeVector scv;
    RotMat    scm;
 
  public Perspective(){
    forward=new ThreeVector( ThreeVector.ZHAT );
    right=new ThreeVector( ThreeVector.XHAT );
    down=new ThreeVector( ThreeVector.YHAT );
    
    loc=new ThreeVector();
    
    A=1.0;
    size=1.0;
    
    x=0;
    y=0;
    changed=true;
   
    scv=new ThreeVector();
    scm=new RotMat();

  }

  //-----------------------------------------------------------------------------

  public void set_positionDefaults(double a,double scale){
    forward.set_equ( ThreeVector.ZHAT );
    right.set_equ( ThreeVector.XHAT );
    down.set_equ( ThreeVector.YHAT );
    loc.set_xyz( 0.0,0.0,-scale/a);
    changed=true;

  }

  public void set_trueScales(int w,int h){
    size=(w+h)/4.0;
    A=size/OBS;
    changed=true;
  }
  
  public void set_scaledScales(int w,int h, double scale,double distance){
    size=(w+h)/4.0;
    A=scale/distance;
    changed=true;
  }
  
  
  public void set_centered(int w,int h){
    x=w/2; y=h/2;
    changed=true;
  }
  
  public void set_scaledDefaults(int w,int h,double scale,double distance){
    set_scaledScales(w,h,scale,distance);
    set_positionDefaults( A, scale );
    set_centered(w,h);
  }
									  
  public void set_trueDefaults(int w,int h,double scale){
    set_trueScales(w,h);
    set_positionDefaults( A,scale);
    set_centered(w,h);
  }


  public final boolean hasChanged(){ return changed; }
  public final void clearChanged(){ changed=false; }

  public ThreeVector getLocation(){ return loc; }
  public ThreeVector getForward(){ return forward; }
  public ThreeVector getRight(){ return right; }
  public ThreeVector getDown(){ return down; }

  public double getViewAngle(){
    return Math.atan(A);
  }


  //-------------------------------------------------------------------------
  public void setLocation(ThreeVector a){
    loc.set_equ(a);
    changed=true;
  }
  public void translate(ThreeVector a){
    loc.plusEquals(a);
    changed=true;
  }
 
  public void translate(int direction,double distance){
    switch(direction){
    case RIGHT: translateRight(distance); break;
    case DOWN: translateDown(distance); break;
    case FORWARD: translateForward(distance); break;
    default: // do nothing
    }
  }
  public void rotateObserver(RotMat r){
    scv.set_equ(loc);
    loc.set_act(r,scv);
    changed=true;
  }
  public final void translateForward(double a){
    scv.set_mul( forward,a);    translate( scv );
  }
  public final void translateRight(double a){
    scv.set_mul( right, a);    translate( scv );
  }
  public final void translateDown(double a){
    scv.set_mul( down,a);    translate(scv);
  }


  public final java.awt.Point getCenter(){
    return new java.awt.Point(x,y);
  }


  public void magnify(double factor){
    A/=factor;
  }

  //-----------------------------------------------------------------
  

  public void crane(RotMat r){
    scv.set_equ( forward );    forward.set_act( r, scv );
    scv.set_equ( right   );      right.set_act( r, scv );
    scv.set_equ( down    );       down.set_act( r, scv );
    changed=true;
  }

  public void crane(int axis, double angle){
    switch(axis){
    case RIGHT: craneUp(angle); break;
    case DOWN: craneRight(angle); break;
    case FORWARD: craneRoll(angle); break;
    default: // do nothing
    }
  }

  public final void craneUp(double ang){
    scm.set_rotationAbout( right,ang);
    crane(scm);
  }
  public final void craneRight(double ang){
    scm.set_rotationAbout( down, ang);
    crane(scm);
  }
  public final void craneRoll(double ang){
    scm.set_rotationAbout(forward,ang);
    crane(scm);
  }
  
  
  public final void getDisplay(Locator L,ThreeVector a){

    scv.set_dif(a,loc);
    double r,f,d;
    f=scv.dot(forward)*A;
    r=scv.dot(right)/f;
    d=scv.dot(down)/f;
    
    L.x=(int)(size*r)+x;
    L.y=(int)(size*d)+y;
    L.depth=size/A/scv.norm();
    if( f <=0.0 || Math.abs(r) > 2.0 || Math.abs(d) > 2.0 ){
      L.depth=Double.POSITIVE_INFINITY;
    }
  }

  public final double getDepth(ThreeVector v){
    scv.set_dif(v,loc);
    return size/scv.norm();
  }
}



