package utils.d3;

import utils.d3.ThreeVector;

public final class RotMat{
  double xx,xy,xz;
  double yx,yy,yz;
  double zx,zy,zz;



  public static final RotMat new_xRotation(double angle){
    RotMat tmp=new RotMat();
    tmp.set_xRotation(angle);
    return tmp;
  }
  public static final RotMat new_yRotation(double angle){
    RotMat tmp=new RotMat();
    tmp.set_yRotation(angle);
    return tmp;
  }
  public static final RotMat new_zRotation(double angle){
    RotMat tmp=new RotMat();
    tmp.set_zRotation(angle);
    return tmp;
  }
  
  public RotMat(){
    xx=1.0; xy=0.0; xz=0.0;
    yx=0.0; yy=1.0; yz=0.0;
    zx=0.0; zy=0.0; zz=1.0;
  }
  public RotMat(RotMat r){
    xx=r.xx; xy=r.xy; xz=r.xz;
    yx=r.yx; yy=r.yy; yz=r.yz;
    zx=r.zx; zy=r.zy; zz=r.zz;
  }

  public void set_Identity(){
    xx=1.0; xy=0.0; xz=0.0;
    yx=0.0; yy=1.0; yz=0.0;
    zx=0.0; zy=0.0; zz=1.0;
  }    

  
  public void set_zRotation(double ang){
    double c=Math.cos(ang);
    double  s=Math.sin(ang);
    xx=c   ;   xy=-s  ;   xz=0.0 ; 
    yx=s   ;   yy=c   ;   yz=0.0 ;
    zx=0.0 ;   zy=0.0 ;   zz=1.0 ;
  }

  public void set_yRotation(double ang){
    double c=Math.cos(ang);
    double  s=Math.sin(ang);
    xx=c   ;   xy=0.0 ;   xz=s   ; 
    yx=0.0 ;   yy=1.0 ;   yz=0.0 ;
    zx=-s  ;   zy=0.0 ;   zz=c   ;
  }

  public void set_xRotation(double ang){
    double c=Math.cos(ang);
    double s=Math.sin(ang);
    xx=1.0;   xy=0.0;    xz=0.0; 
    yx=0.0;   yy=c  ;    yz=-s;
    zx=0.0;   zy=s  ;    zz=c ;
  }
  
  
  public void set_rotationAbout(ThreeVector axis,double ang){
    double norm=axis.norm();
    if( ang==0.0 || norm==0.0 ){
      set_Identity();
      return;
    }
    ThreeVector n=axis.times(1.0/norm);
    double c=Math.cos(ang);
    double onemc=1.0-c;
    double s=-Math.sin(ang);
    xx=n.x*n.x*onemc+c;   
    xy=n.z*s+n.x*n.y*onemc;   
    xz=-n.y*s+n.x*n.z*onemc; 
    yx=-n.z*s+n.y*n.x*onemc; 
    yy=n.y*n.y*onemc+c;
    yz=n.x*s+n.y*n.z*onemc;
    zx=n.y*s+n.z*n.x*onemc;
    zy=-n.x*s+n.z*n.y*onemc;
    zz=n.z*n.z*onemc+c;
  }

  public void set_equ(RotMat r){
    xx=r.xx; xy=r.xy; xz=r.xz;
    yx=r.yx; yy=r.yy; yz=r.yz;
    zx=r.zx; zy=r.zy; zz=r.zz;
  }

  public void set_com(RotMat a,RotMat b){
    xx=a.xx*b.xx + a.xy*b.yx + a.xz*b.zx;   
    xy=a.xx*b.xy + a.xy*b.yy + a.xz*b.zy;   
    xz=a.xx*b.xz + a.xy*b.yz + a.xz*b.zz; 
    yx=a.yx*b.xx + a.yy*b.yx + a.yz*b.zx;  
    yy=a.yx*b.xy + a.yy*b.yy + a.yz*b.zy;
    yz=a.yx*b.xz + a.yy*b.yz + a.yz*b.zz;
    zx=a.zx*b.xx + a.zy*b.yx + a.zz*b.zx;
    zy=a.zx*b.xy + a.zy*b.yy + a.zz*b.zy;
    zz=a.zx*b.xz + a.zy*b.yz + a.zz*b.zz;
  }

  public void set_inv(RotMat a){
    set_equ(a);
    double dum;
    //exchange(xy,yx);
    dum=xy; xy=yx; yx=dum;
    //exchange(zx,xz);
    dum=zx; zx=xz; xz=dum;
    //exchange(yz,zy);
    dum=yz; yz=zy; zy=dum;
  }


  public RotMat actingOn(RotMat that){
    RotMat tmp=new RotMat();
    tmp.set_com(this,that);
    return tmp;
  }

  

  public RotMat inverse(){
    RotMat tmp=new RotMat(this);
    tmp.set_inv(this);
    return tmp;
  }

  public ThreeVector actingOn(ThreeVector a){
    ThreeVector tmp=new ThreeVector();
    tmp.set_act(this,a);
    return tmp;
  }

  public String toString(){
    String tmp=
      "("+xx+","+xy+","+xz+")"+
      "("+yx+","+yy+","+yz+")"+
      "("+zx+","+zy+","+zz+")";
    return tmp;
  }
}//end of class rotmat









