package utils.d3;

/** @author Dave McNamara 
    A class to handle three dimensional vectors
    4/2#/97 ----
	
*/


public final class ThreeVector{
  /** define the three unit vectors */
  double x,y,z;
  
  public static final ThreeVector XHAT=new ThreeVector(1.0,0.0,0.0);
  public static final ThreeVector YHAT=new ThreeVector(0.0,1.0,0.0);
  public static final ThreeVector ZHAT=new ThreeVector(0.0,0.0,1.0);
  public static final ThreeVector ZERO=new ThreeVector(0.0,0.0,0.0);
  

  public ThreeVector(){
    x=0.0; y=0.0; z=0.0;
  }
  
  public ThreeVector(ThreeVector v){
    x=v.x; y=v.y; z=v.z;
  }
  
  public static ThreeVector new_xyz(double xx,double yy,double zz){
    return new ThreeVector(xx,yy,zz);
  }
  public static ThreeVector new_rtp(double r,double th,double ph){
    ThreeVector tmp=new ThreeVector();
    tmp.set_rtp(r,th,ph);
    return tmp;
  }
  
  private ThreeVector(double xx,double yy,double zz){
    x=xx; y=yy; z=zz;
  }
  
  
  public double xComponent(){ return x; }
  public double yComponent(){ return y; }
  public double zComponent(){ return z; }

  public void set_equ(ThreeVector that){
    this.x=that.x; this.y=that.y; this.z=that.z;
  }

  public void set_nul(){
    x=0.0; y=0.0; z=0.0;
  }


  public void set_rtp(double r,double theta,double phi){
    z=r*Math.cos(theta);
    y=r*Math.sin(theta)*Math.sin(phi);
    x=r*Math.sin(theta)*Math.cos(phi);
  }

  public void set_xyz(double xx,double yy,double zz){
    x=xx; y=yy; z=zz;
  }
  
  public ThreeVector plus(ThreeVector that){
    return new ThreeVector(
			   this.x+that.x,
			   this.y+that.y,
			   this.z+that.z );
  }
  public ThreeVector minus(ThreeVector that){
    return new ThreeVector(
			   this.x-that.x,
			   this.y-that.y,
			   this.z-that.z );
  }
  public ThreeVector times(double a){
    return new ThreeVector( x*a, y*a, z*a );
  }


  public void plusEquals(ThreeVector that){
    this.x+=that.x; this.y+=that.y; this.z+=that.z;
  }
  public void minusEquals(ThreeVector that){
    this.x-=that.x; this.y-=that.y; this.z-=that.z;
  }
  public void timesEquals(double a){
    x*=a; y*=a; z*=a;
  }
  
  public void plusEquals_xyz(double xx,double yy,double zz){
    x+=xx; y+=yy; z+=zz;
  }

  public double dot(ThreeVector that){
    return this.x*that.x+this.y*that.y+this.z*that.z;
  }

  public void set_sum(ThreeVector a,ThreeVector b){
    x=a.x+b.x; y=a.y+b.y; z=a.z+b.z;
  }
  public void set_dif(ThreeVector a,ThreeVector b){
    x=a.x-b.x; y=a.y-b.y; z=a.z-b.z;
  }
  public void set_mul(ThreeVector a,double u){
    x=a.x*u; y=a.y*u; z=a.z*u;
  }
  public void set_cro(ThreeVector a,ThreeVector b){
    x=a.y*b.z-a.z*b.y;
    y=a.z*b.x-a.x*b.z;
    z=a.x*b.y-a.y*b.x;
  }
  

  public void set_act(RotMat r,ThreeVector a){
    x=r.xx*a.x + r.xy*a.y + r.xz*a.z;
    y=r.yx*a.x + r.yy*a.y + r.yz*a.z;
    z=r.zx*a.x + r.zy*a.y + r.zz*a.z;
  }


  public ThreeVector cross(ThreeVector that){
    ThreeVector tmp=new ThreeVector();
    tmp.set_cro(this,that);
    return tmp;
  }
  
  public double norm(){
    return Math.sqrt( x*x+y*y+z*z);
  }
  
  public ThreeVector normalized(){
    ThreeVector tmp=new ThreeVector(this);
    tmp.normalize();
    return tmp;
  }
  public void normalize(){
    this.timesEquals( 1.0/this.norm());
  }

  
  public String toString(){
    return "("+String.valueOf(x)+","+String.valueOf(y)+","+
      String.valueOf(z)+")";
  }

}









