package utils.d3.primitive;

import utils.d3.Perspective;
import java.awt.Graphics;


public class VectorOfPrimitives{
  
  GraphicsPrimitive[] stuff;
  int current;
  int max;
  int block;
  
  public VectorOfPrimitives(int initial,int bl){
    max=Math.max(initial,2);
    current=0;
    stuff=new GraphicsPrimitive[max];
    block=Math.max(bl,2);
  }

  public VectorOfPrimitives(){
    this(10,10);
  }
  
  public int add(GraphicsPrimitive gp){
    augmentCapacity(1);
    stuff[current]=gp;
    ++current;
    return current-1;
  }
  
  public void ensure(int num){
    if( num > max ) augmentCapacity( num-max );
  }
  
  void augmentCapacity(int increment){
    if( current+increment > max ){
      GraphicsPrimitive[] old=stuff;
      max=Math.max( max+block, current+increment);
      stuff=new GraphicsPrimitive[max];
      System.arraycopy(old,0,stuff,0,current);
    }
  }

  public GraphicsPrimitive primitiveAt(int i){ return stuff[i]; }
  public int size(){ return current; }
  
  public int indexOf(GraphicsPrimitive gp){
    for(int i=0;i<current; ++i){
      if( stuff[i].equals(gp) ) return i;
    }
    return -1;
  }
  
  public GraphicsPrimitive removeAt(int i){
    GraphicsPrimitive tmp=stuff[i];
    int j=current-i-1;
    if(j>0){
      System.arraycopy( stuff,i+1,stuff,i,j);
    }
    --current;
    stuff[current]=null;
    return tmp;
  }

  public void remove(GraphicsPrimitive gp){
    int index=indexOf(gp);
    if( index >= 0){
      removeAt(index);
    }
  }
  public void updateDisplay(Perspective p){
    for(int i=0;i<current;++i){
      stuff[i].updateDisplay(p);
    }
    sort();
  }
  public void draw(Graphics g){
    for(int i=0; 
	i< current && !Double.isInfinite( stuff[i].getDepth()) ; ++i){
      stuff[i].draw(g);
    }
  }

  public GraphicsPrimitive findUnder(int x,int y){
    double depth=stuff[0].getDepth();
    for(int i=0;i<current && !Double.isInfinite(depth) ; ++i){
      if( stuff[i].isHitBy(x,y) ) return stuff[i];
    }
    return null;
  }
  
  public int indexUnder(int x,int y){
    for(int i=0; i<current && !Double.isInfinite( stuff[i].getDepth()) ; ++i){
      if( stuff[i].isHitBy(x,y) ) return i;
    }
    return -1;
  }
  
  public void sort(){
    GraphicsPrimitive tmp;
    boolean flag=true;
    int max=current;
    while(flag){
      flag=false;
      for(int i=1; i< max ; ++i){
	if( stuff[i].getDepth() < stuff[i-1].getDepth() ){
	  tmp=stuff[i];
	  stuff[i]=stuff[i-1];
	  stuff[i-1]=tmp;
	  flag=true;
	}
      }
    }
  }

}
