package utils.tango;

import consts.*;
import interfaces.*;
import messages.AppEventMessage;

import java.awt.Event;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;


public class TangoHelper{
  
  public static final int NULL = 0;
  public static final int BOOLEAN=1;
  public static final int CHAR=2;
  public static final int INT=4;
  public static final int LONG=5;
  public static final int FLOAT=6;
  public static final int DOUBLE=7;
  public static final int STRING=8;


  public static AppEventMessage makeAppEventMessage(Event e){

    try{
      ByteArrayOutputStream baos=new ByteArrayOutputStream();
      DataOutputStream dos=new DataOutputStream(baos);
      
      dos.writeLong(e.when);
      dos.writeInt(e.id);
      dos.writeInt(e.x);
      dos.writeInt(e.y);
      dos.writeInt(e.key);
      dos.writeInt(e.modifiers);
      
      if( e.arg == null ){
	dos.writeInt( NULL );
      }
      else{
	if( e.arg instanceof Boolean ){
	  dos.writeInt( BOOLEAN );
	  dos.writeBoolean( ((Boolean)e.arg).booleanValue());
	}
	if( e.arg instanceof Character ){
	  dos.writeInt( CHAR );
	  dos.writeChar( ((Character)e.arg).charValue());
	}
	if( e.arg instanceof Integer ){
	  dos.writeInt( INT );
	  dos.writeInt( ((Integer)e.arg).intValue());
	}
	if( e.arg instanceof Long ){
	  dos.writeInt( LONG );
	  dos.writeLong( ((Long)e.arg).longValue());
	}
	if( e.arg instanceof Float ){
	  dos.writeInt( FLOAT );
	  dos.writeFloat( ((Float)e.arg).floatValue());
	}
	if( e.arg instanceof Double){
	  dos.writeInt( DOUBLE );
	  dos.writeDouble( ((Double)e.arg).doubleValue());
	}
	if( e.arg instanceof String ){
	  dos.writeInt( STRING );
	  dos.writeUTF( (String)e.arg);
	}
      }
      
      AppEventMessage aem=new AppEventMessage( Const.DATA, baos.toByteArray());
      dos.close();
      baos.close();
      return aem;
    }
    catch( IOException ioe){
      System.out.println("Exception while transforming an Event ");
      return null;
    }
  }

    


  public static Event makeEvent(AppEventMessage aem){
    Event tmp=new Event( null,0,null);
    
    try{
      ByteArrayInputStream bais=new ByteArrayInputStream( aem.getData());
      DataInputStream dis=new DataInputStream(bais);
      
      tmp.when=dis.readLong();
      tmp.id=dis.readInt();
      tmp.x=dis.readInt();
      tmp.y=dis.readInt();
      tmp.key=dis.readInt();
      tmp.modifiers=dis.readInt();

      int type=dis.readInt();
      
      switch( type ){
      case NULL:  /* do nothing */ break;
      case BOOLEAN: tmp.arg= new Boolean( dis.readBoolean() ); break;
      case CHAR: tmp.arg=new Character( dis.readChar() ); break;
      case INT: tmp.arg=new Integer( dis.readInt()); break;
      case LONG: tmp.arg=new Long( dis.readLong()); break;
      case FLOAT: tmp.arg=new Float( dis.readFloat()); break;
      case DOUBLE: tmp.arg=new Double( dis.readDouble()); break;
      case STRING: tmp.arg= dis.readUTF();
      default:
	System.out.println("Unknown arg type index: "+type);
      }
    }
    catch(IOException ioe){
      System.out.println("error recovering Event from AppEventMessage");
    }
    return tmp;
  }
  
}
