package utils.timers;

import utils.timers.Dynamic;

/**
	A Thread which throws equally sized timesteps at Dynamic
	objects.  It tries to keep the realtime intervals between
	kicks constant, and near a user defined level.  Kicks
	maxKickCount times than stops.  Can be reset by kicking the
	kicker with a call to kick().  Goes until explicitly stopped
	if maxKickCount == 0.
 
	@see Dynamic
**/

public class FixedKicker extends Thread{
    Object lock;

    /** The minimum amount of time that the thread will actually spend 
	sleeping per kick. */
    public static final long MIN_SLEEP=30;
    
    /** Simulation time per tick. */
    public static double dt=0.01;
    
    /** The objects to be kicked. */
    Dynamic driven[] = new Dynamic[10];
    
    /** The period between kicks for this object. */
    int period[] = new int[10];
    
    /** Whether this object is active or not. */
    boolean active[] = new boolean[10];

    /** Length of the arrays. */
    int length=10;                     
    
    /** Number of entries in the arrays. */
    int entries=0;          
    
    /** How many kicks we;ve sent out. */
    public int kickCount=0;                   
    
    /** The number of kicks we will send. */
    int maxKickCount=0; 
    
    /** Whether or not to continue kicking. */
    boolean dontstop;                   
    
    /** The target amount of time to wait between timesteps. */
    long sl;                            
                                        
    FixedKicker(){
	super("Flow of Time");
	lock = new Object();
    }

    /**
       @param d  the @see Dynamic object to be kicked
       @param s  desired interval between hitting the top of the run loop 
    **/

    public FixedKicker(Dynamic d,long s){
	super("Flow of Time");
	lock = new Object();
	driven[0]=d;
	period[0]=1;
	active[0]=true;
	sl=s;
    }

    public FixedKicker(int mkc){
	super("Flow of Time");
	lock = new Object();
	maxKickCount = mkc;
    }

    public void setActive(Dynamic d, boolean a){
	for(int i=0; i<entries; ++i){
	    if(driven[i] == d){
		active[i] = a;
	    }
	}
    }
		
		

    public void addDriven(Dynamic d, int p, boolean a){
	driven[entries] = d;
	period[entries] = p;
	active[entries] = a;
	entries++;
	if(entries == length){
	    Dynamic oldd[] = driven;
	    int oldp[] = period;
	    boolean olda[] = active;
	    driven = new Dynamic[2*length];
	    period = new int[2*length];
	    active = new boolean[2*length];
	    for(int i=0; i<entries;++i){
		driven[i] = oldd[i];
		period[i] = oldp[i];
		active[i] = olda[i];
	    }
	    oldd =  null;
	    oldp = null;
	    olda = null;
	    length *= 2;
	}
    }
		
    public void setMaxKickCount(int mkc){
	maxKickCount = mkc;
    }

    public void kick(){
	//Finish any remaining kicks immediately
	synchronized(lock){
	    while(kickCount < maxKickCount){
		kickCount++;
		for(int i = 0; i<entries;++i){
		    if(kickCount%period[i] == 0 && active[i]){
			driven[i].timestep(dt);
		    }
		}
	    }
	}
    }

    public void again(){
	synchronized(lock){
	    kickCount = 0;
	}	
	if(!isAlive()) start();
    }

    public void start(){
	dontstop=true;
	super.start();
    }

    public void run(){
	long thistime;
	long lasttime;
	double eltime;
  
	lasttime=System.currentTimeMillis();

	while(true){
	    synchronized(lock){
		if(dontstop && ( (kickCount < maxKickCount) || (maxKickCount == 0) )){
		    thistime=System.currentTimeMillis();
		    
		    ++kickCount;
		    for(int i = 0; i<entries;++i){
			if(kickCount%period[i] == 0 && active[i]){
			    driven[i].timestep(dt);
			}
		    }
		    lasttime=thistime;
		    try{
			long zzz=Math.max(MIN_SLEEP,sl-System.currentTimeMillis()+thistime);
			//System.out.println(zzz);
			sleep( zzz );
		    }
		    catch(InterruptedException ie){ return; }
		}
	    }
	}
    }

    /** 
	the nice way to stop the thread.  i.e. the thread will drop out
	of the run method.  this prevents "IllegalThreadStateException"s
	if the Dynamic object is used again
    **/  

    public void pleaseStop(){
	dontstop=false;
    }
    public void setSleepTime(long new_sleep){ sl=new_sleep; }
}

