package utils.timers;

import java.awt.Panel;
import java.awt.Event;
import java.awt.Button;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import utils.GUI.LabelledScroller;
import utils.GUI.GridBagHelpers;



/******************************************************************
 * provides a graphical control panel for a FixedKicker thread
 * allows the user to adjust the thread parameters and 
 * stop/start the thread.
 ******************************************************************/
public class FixedKickerControl extends Panel{
  public static final String PAU="Pause";
  public static final String CON="Continue";


  LabelledScroller max;
  LabelledScroller sleep;
  Button pause;
  Button conti;

  Dynamic kicked;
  FixedKicker time;

  

  /**
   * construct by specifying what will be kicked
   **/

  public FixedKickerControl(Dynamic kiced){
    max=new LabelledScroller("Max dt",0.0,0.1);
    max.setValue(0.01);

    sleep=new LabelledScroller("Sleep Time",0.0,100.0);
    sleep.setValue( 50.0 );

    pause=new Button(PAU);
    conti=new Button(CON);
    kicked=kiced;
    time=null;
    
    setLayout( new GridBagLayout());
    GridBagHelpers.insert(this,max,
			  1,1,
			  2,1,
			  GridBagConstraints.HORIZONTAL,
			  GridBagConstraints.CENTER,
			  0,0,
			  1.0,1.0,
			  4,0,4,0);
    GridBagHelpers.insert(this,sleep,
			  1,2,
			  2,1,
			  GridBagConstraints.HORIZONTAL,
			  GridBagConstraints.CENTER,
			  0,0,
			  1.0,1.0,
			  4,0,4,0);
    GridBagHelpers.insert(this,conti,
			  1,3,
			  1,1,
			  4,0,4,0,
			  GridBagConstraints.CENTER);
    GridBagHelpers.insert(this,pause,
			  2,4,
			  1,1,
			  4,0,4,0,
			  GridBagConstraints.CENTER);
  }


  public boolean action(Event e,Object arg){
    if( e.target instanceof LabelledScroller){
      return scrollerAction(e,arg);
    }
    else{
      if( PAU.equals(arg) && time!=null ){
	time.pleaseStop();
	try{
	  time.join();
	}
	catch(InterruptedException ie){ /* do nothing */ }
	time=null;
	return true;
      }
      else if( CON.equals(arg) && time==null){
	time=constructThread();
	time.start();
	return true;
      }
      else return false;
    }
  }

  public boolean scrollerAction(Event e,Object arg){
    // if our thread is not alive we don't need to do anything
    if( time==null || ! time.isAlive()) return true;
    
    double value=( (Double)e.arg).doubleValue();
    
    if( e.target.equals(max)){
      FixedKicker.dt=value;
      return true;
    }
    else if( e.target.equals(sleep)){
      time.setSleepTime( Math.round(value) );
      return true;
    }
    else return false;
  }

  FixedKicker constructThread(){ 
    FixedKicker tmp=new FixedKicker(kicked, 
				    Math.round(sleep.getValue()) ); 
    return tmp; 
  }


  /**
   * set the range of allowed sleep times
   **/
  public void setSleepRange(long sl_min,long sl_max){
    sleep.setRange( (double)sl_min,(double)sl_max);
  }
  /**
   * set the range of allowed time-step intervals
   **/
  public void setDTRange(double min_dt,double max_dt){
    max.setRange( min_dt, max_dt );
  }
  
  public double getMinDT(){ return max.getMinValue(); }
  public double getMaxDT(){ return max.getMaxValue(); }
  public double getMinSleep(){ return sleep.getMinValue(); }
  public double getMaxSleep(){ return sleep.getMaxValue(); }

  public double getDT(){ return max.getValue(); }
  public long   getSleep(){ return (long)sleep.getValue(); }

}












