package utils.timers;

import utils.timers.Dynamic;

/**
	a Thread which throws variable sized timesteps at a Dynamic
           object.
        It tries to keep the realtime intervals between kicks constant,
        and near a user defined level.
        It the size of the timestep[simulation units] is some multiple
        of the actual time elapsed since the last kick.

	@see Dynamic

**/

public class TimeKicker extends Thread{
  /** the minimum amount of time that the thread will actually spend sleeping
      per kick **/
  public static final long MIN_SLEEP=30;

  Dynamic driven;
  double  factor;
  boolean dontstop;
  double max_dt;
  long sl;        // the target amount of time to wait between timesteps


TimeKicker(){  super("Flow of Time"); }

/**
@param d  the @see Dynamic object to be kicked
@param f  how much to scale the actul time(in ms) to get simulation time
@param s  desired interval between hitting the top of the run loop 
**/

public TimeKicker(Dynamic d,double f,long s){
  super("Flow of Time");
  driven=d;
  factor=f;
  sl=s;
  max_dt=Double.POSITIVE_INFINITY;
}

public void start(){
  dontstop=true;
  super.start();
}

public void run(){
  long thistime;
  long lasttime;
  double eltime;
  
  lasttime=System.currentTimeMillis();

  while( dontstop ){
   
    thistime=System.currentTimeMillis();
    
    eltime=factor*(thistime-lasttime);
    eltime=Math.min( eltime, max_dt);
    driven.timestep(eltime);
    lasttime=thistime;
    try{
      sleep( Math.max( MIN_SLEEP, sl-System.currentTimeMillis()+thistime));
    }
    catch(InterruptedException ie){ return; }
  }
}

/** 
the nice way to stop the thread.  i.e. the thread will drop out
of the run method.  this prevents "IllegalThreadStateException"s
if the Dynamic object is used again
**/  

public void pleaseStop(){
  dontstop=false;
}
  public void setMaxDT(double new_max){ max_dt=new_max; }
  public void setTimeFactor(double new_factor){ factor=new_factor;}
  public void setSleepTime(long new_sleep){ sl=new_sleep; }
}

