package utils.timers;

import java.awt.Panel;
import java.awt.Event;
import java.awt.Button;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import utils.GUI.LabelledScroller;
import utils.GUI.GridBagHelpers;



/********************************************************
 * provides a GUI to control the parameters of a TimeKicker thread.
 * @see TimeKicker
 ********************************************************/
public class TimeKickerControl extends Panel{
  public static final String PAU="Pause";
  public static final String CON="Continue";


  LabelledScroller factor;
  LabelledScroller max;
  LabelledScroller sleep;
  Button pause;
  Button conti;

  Dynamic kicked;
  TimeKicker time;

  


  public TimeKickerControl(Dynamic kiced){
    factor=new LabelledScroller("Time Factor",0.0,0.2);
    factor.setValue( 0.03 );
    max=new LabelledScroller("Max dt",0.0,1.0);
    max.setValue(1.0);

    sleep=new LabelledScroller("Sleep Time",0.0,100.0);
    sleep.setValue( 50.0 );

    pause=new Button(PAU);
    conti=new Button(CON);
    kicked=kiced;
    time=null;
    
    setLayout( new GridBagLayout());
    
    GridBagHelpers.insert(this,factor,
			  1,1,
			  2,1,
			  GridBagConstraints.HORIZONTAL,
			  GridBagConstraints.CENTER,
			  0,0,
			  1.0,1.0,
			  4,0,4,0);
			  
    GridBagHelpers.insert(this,max,
			  1,2,
			  2,1,
			  GridBagConstraints.HORIZONTAL,
			  GridBagConstraints.CENTER,
			  0,0,
			  1.0,1.0,
			  4,0,4,0);
    GridBagHelpers.insert(this,sleep,
			  1,3,
			  2,1,
			  GridBagConstraints.HORIZONTAL,
			  GridBagConstraints.CENTER,
			  0,0,
			  1.0,1.0,
			  4,0,4,0);
    GridBagHelpers.insert(this,conti,
			  1,4,
			  1,1,
			  4,0,4,0,
			  GridBagConstraints.CENTER);
    GridBagHelpers.insert(this,pause,
			  2,4,
			  1,1,
			  4,0,4,0,
			  GridBagConstraints.CENTER);
  }


  public boolean action(Event e,Object arg){
    if( e.target instanceof LabelledScroller){
      return scrollerAction(e,arg);
    }
    else{
      if( PAU.equals(arg) && time!=null ){
	time.pleaseStop();
	try{
	  time.join();
	}
	catch(InterruptedException ie){ /* do nothing */ }
	time=null;
	return true;
      }
      else if( CON.equals(arg) && time==null){
	time=constructThread();
	time.start();
	return true;
      }
      else return false;
    }
  }

  public boolean scrollerAction(Event e,Object arg){
    // if our thread is not alive we don't need to do anything
    if( time==null || ! time.isAlive()) return true;
    
    double value=( (Double)e.arg).doubleValue();
    
    if( e.target.equals(factor) ){
      time.setTimeFactor(value);
      return true;
    }
    else if( e.target.equals(max)){
      time.setMaxDT(value);
      return true;
    }
    else if( e.target.equals(sleep)){
      time.setSleepTime( Math.round(value) );
      return true;
    }
    else return false;
  }

  TimeKicker constructThread(){
    TimeKicker tmp=new TimeKicker(kicked,
				  factor.getValue(),
				  Math.round(sleep.getValue()) );
    tmp.setMaxDT(max.getValue());
    return tmp;
  }


  public void setFactorRange(double min_f,double max_f){
    factor.setRange(min_f,max_f);
  }
  public void setSleepRange(long sl_min,long sl_max){
    sleep.setRange( (double)sl_min,(double)sl_max);
  }
  public void setMaxDTRange(double min_dt,double max_dt){
    max.setRange( min_dt, max_dt );
  }

  public double getMinFactor(){ return factor.getMinValue(); }
  public double getMaxFactor(){ return factor.getMaxValue(); }
  
  public double getMaxMaxDT(){ return max.getMaxValue(); }
  public double getMinMaxDT(){ return max.getMinValue(); }
  
  public double getMaxSleep(){ return sleep.getMaxValue(); }
  public double getMinSleep(){ return sleep.getMinValue(); }

  public double getFactor(){ return factor.getValue(); }
  public double getMaxDT(){ return max.getValue(); }
  public long   getSleep(){ return (long)sleep.getValue(); }

}












